/* $Revision: 2.0 $ $Date: 2000/04/24 17:39:16 $ */
#include <math.h>
#include "mex.h"

#define GET_TOTAL_ELEMENTS(a)  (mxGetM(a) * mxGetN(a)) 

#ifndef HUGE
#define HUGE HUGE_VAL
#endif

/* Input Arguments */

#define	IN	prhs[0]
#define DIM	prhs[1]
#define W	prhs[2]

/* Output Arguments */

#define	OUT	plhs[0]	


static double
logsum(double x, double y)
{
  if (x==-HUGE)
    return y;

  if (y==-HUGE)
    return x;

  return ( (x>y) ? ( x + log( 1. + exp(y-x) ) ) : ( y + log( 1. + exp(x-y) ) ) );
}


void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*pIn, *pIn1, *pIn2, *pDim, *pOut, *pW, w;
  int		N_In, N_Out, N1, N2, N3, nDimIn, nDimOut, dim;
  int		*dimsOut, *cumDim;
  int		i, j, k, l;
  const int	*dimsIn;

  /* Check for proper number of arguments */
  
  if (nrhs < 1 || nrhs > 3)  {
    mexErrMsgTxt("Requires one to three input arguments.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("Requires one output argument.");
  }
  
  /* Get input pointers */
  pIn  = mxGetPr( IN );

  if (nrhs>1)
    dim = (int)(*mxGetPr( DIM ) - 1);
  else
    dim = 0;

  if (nrhs>2)
    pW = mxGetPr( W );
  else
    pW = NULL;
  
  /* Get IN dimensions */
  nDimIn = mxGetNumberOfDimensions(IN);
  dimsIn = mxGetDimensions(IN);

  if (dim<0) 
    mexErrMsgTxt("Dimension argument must be a positive integer.");
  else if (dim>nDimIn-1)
    mexErrMsgTxt("Dimension argument must be within input dimensions.");    

  if (pW && dimsIn[dim] != GET_TOTAL_ELEMENTS(W)) {
    mexErrMsgTxt("Input and weight dimensions do not agree.");
  }

  /* Allocate storage for dimsOut */
  dimsOut = mxCalloc(nDimIn,sizeof(int));
  cumDim = mxCalloc(nDimIn+1,sizeof(int));

  /* Copy dimsIn into dimsOut, skipping the dim-th dimension */
  for (i=0;i<nDimIn;i++)
    if (i==dim)
      dimsOut[i] = 1;
    else
      dimsOut[i] = dimsIn[i];

  /* Find cumulative product of dimensions */
  for (i=0,cumDim[0]=1;i<nDimIn;i++)
    cumDim[i+1] = cumDim[i]*dimsIn[i];

  /* Create outputs */
  OUT	= mxCreateNumericArray(nDimIn, dimsOut, mxDOUBLE_CLASS, mxREAL);
  N_Out = GET_TOTAL_ELEMENTS( OUT );

  /* Set output pointers */
  pOut = mxGetPr( OUT );

  /* Reduce problem to summing of a 3-D array */
  N1 = cumDim[dim]; /* number of elements in all dimension up to 'dim' */
  N2 = dimsIn[dim]; /* number of elements in dimension 'dim' */
  N3 = N_Out/cumDim[dim]; /* number of elements in dimension after 'dim' */

  /* initialize */
  for (l=0;l<N1*N3;l++)	
    pOut[l] = -HUGE;

  /* logsum along dim */
  for (i=0;i<N2;i++) {
    
    if (pW)
      w = pW[i];
    else
      w = 1;
    
    if (w) {

      pIn1 = pIn + i*cumDim[dim];
      
      for (j=0,l=0;j<N3;j++) {
	
	pIn2 = pIn1 + j*cumDim[dim+1];

	for (k=0;k<N1;k++,l++) {
	  /* pOut[l] += pIn[i*cumDim[dim] + j*cumDim[dim+1] + k]*pW[i] ; */
	  pOut[l] = logsum( pOut[l], w*pIn2[k] );
	}
      }  

    }

  }
  
  mxFree(dimsOut);
  mxFree(cumDim);

  return;
}

