/* $Revision: 2.0 $ $Date: 2000/04/24 17:39:17 $ */

#include "math_util.h"

#ifdef USE_MATLAB_MEMFUN
#define calloc	mxCalloc
#define free	mxFree
#define realloc	mxRealloc
#include <mex.h>
#endif

/* Memory stuff --- should use heap */
#define VECTOR(v,N) \
static int dim = 0; \
static double *v = NULL; \
if (dim<N) { dim = N; v = realloc(v,N*sizeof(double)); } else if (N==0) { dim = 0; if (v) { free(v); v = NULL; } }

void
emf_clean()
{

  copy_(NULL,0);
  neg_(NULL,0);
  scale_(NULL,0,0);
  trans_(NULL,0,0);
  sum_(NULL,NULL,0);
  diff_(NULL,NULL,0);
  prod_(NULL,NULL,0,0,0);
  dot_(NULL,NULL,0,0,0);
  out_(NULL,NULL,0,0,0);
  oqprod_(NULL,NULL,NULL,0,0);
  iqprod(NULL,NULL,NULL,0,0,NULL);
  iqprod_(NULL,NULL,NULL,0,0);
  sinv_(NULL,0);

}

/* LOG/LIN calls */

double
logSum(double x, double y)
{
  if (!finite(x) )
    return y;

  if (!finite(y) )
    return x;

  return ( (x>y) ? 
	   ( x + log( 1. + exp(y-x) ) ) : 
	   ( y + log( 1. + exp(x-y) ) ) );
}

void
lin2log(double *x, int N)
{
  int i, j;
  
  for(i=0;i<N;i++,x++)
    if (*x)
      *x = log( *x );
    else
      *x = -HUGE;
}

void
log2lin(double *x, int N)
{
  int i, j;
  
  for(i=0;i<N;i++,x++)
    if (finite(*x))
      *x = exp( *x );
    else
      *x = 0;
}


/*
 * Basic vector & matrix routines
 *
 */
void
printm( double *x, int N, int M)
{
  int i,j;
  for(i=0;i<N;i++,printf("\n"))
    for(j=0;j<M;j++,x++)
      printf("%f ", *x);
  printf("\n");
}

void
copy( double *x, int N, double *y)
{
  int i;

  for (i=0;i<N;i++)
    y[i] = x[i];
}
  
double*
copy_(double *x, int N)
{
  VECTOR(y,N);

  copy(x,N,y);

  return(y);
  
}

/* y = -x */
void
neg(double *x, int N, double *y)
{
  int i;
  for(i=0;i<N;i++)
    y[i] = -x[i];
}

double*
neg_(double *x, int N)
{
  VECTOR(y,N);

  neg(x,N,y);

  return(y);
  
}

/* y = alpha*x */
void
scale(double *x, double alpha, int N, double *y)
{
  int i;
  for(i=0;i<N;i++)
    y[i] = alpha*x[i];
}

double*
scale_(double *x, double alpha, int N)
{
  VECTOR(y,N);

  scale(x,alpha,N,y);

  return(y);
}

/* y = x' */
void
trans( double *x, int N, int M, double *y)
{
  int i,j;
  for(j=0;j<M;j++)
    for(i=0;i<N;i++)
      y[j+i*M] = x[i+j*N];
}

double*
trans_(double *x, int N, int M)
{
  VECTOR(y,N*M);
  
  trans(x,N,M,y);

  return(y);
}

/* y = x'*W*x */
double
wnorm( double *W, double *x, int N )
{
  int i, j;
  double tmp, y = 0;
  
  for (i=0;i<N;i++) {
    tmp = 0;
    for (j=0;j<N;j++)
      tmp += W[i+N*j]*x[j];
    y += tmp*x[i];
  }

  return y;
}

double
trace( double *x, int N )
{
  int i;
  double sum = 0;

  for (i=0;i<N;i++,x+=(N+1))
    sum += *x;

  return(sum);
}

/* z = x+y */
void
sum(double *x, double *y, int N, double *z)
{
  int i;
  for (i=0;i<N;i++)
    z[i] = x[i]+y[i];
}

double*
sum_(double *x, double *y, int N)
{
  VECTOR(z,N);
  
  sum(x,y,N,z);

  return(z);
}


/* z = x-y */
void
diff(double *x, double *y, int N, double *z)
{
  int i;
  for (i=0;i<N;i++)
    z[i] = x[i]-y[i];
}

double*
diff_(double *x, double *y, int N)
{
  VECTOR(z,N);
  
  diff(x,y,N,z);

  return(z);
}

/* z = x*y */
void
prod(double *x, double *y, int N, int L, int M, double *z)
{
  int i, j, k;
  for (j=0;j<M;j++)
    for(i=0;i<N;i++) {
      for(z[i+j*N]=0,k=0;k<L;k++)
	z[i+j*N] += x[i+k*N]*y[k+j*L];
      if (isnan(z[i+j*N]))
	z[i+j*N] = 1;
    }
	  
}

double*
prod_(double *x, double *y, int N, int L, int M)
{
  VECTOR(z,N*M);
  
  prod(x,y,N,L,M,z);

  return(z);
}

/* z = x'*y */
void
dot(double *x, double *y, int N, int L, int M, double *z)
{
  int i, j, k;
  for (j=0;j<M;j++)
    for(i=0;i<N;i++)
      for(z[i+j*N]=0,k=0;k<L;k++)
	z[i+j*N] += x[k+i*L]*y[k+j*L];  
}

double*
dot_(double *x, double *y, int N, int L, int M)
{
  VECTOR(z,N*M);
  
  dot(x,y,N,L,M,z);

  return(z);
}

/* z = x*y' */
void
out(double *x, double *y, int N, int L, int M, double *z)
{
  int i, j, k;
  for (j=0;j<M;j++)
    for(i=0;i<N;i++)
      for(z[i+j*N]=0,k=0;k<L;k++)
	z[i+j*N] += x[i+k*N]*y[j+k*M];  
}

double*
out_(double *x, double *y, int N, int L, int M)
{
  VECTOR(z,N*M);
  
  out(x,y,N,L,M,z);

  return(z);
}


void
oqprod(double *A, double *x, double *B, int N, int M, double *y)
{
  int i;

  for (i=0;i<N*N;i++)
    y[i] = 0;
  
  oquadop( 1, A,x,B, N,M,M, y );

}

double*
oqprod_(double *A, double *x, double *B, int N, int M)
{
  VECTOR(y,N*N);

  oqprod(A,x,B, N,M, y );

  return(y);
}


void
iqprod(double *A, double *x, double *B, int N, int M, double *y)
{
  int i;
  VECTOR(At,N*M);

  for (i=0;i<N*N;i++)
    y[i] = 0;

  trans(A,M,N,At);

  oquadop( 1, At,x,B, N,M,M, y );

}

double*
iqprod_(double *A, double *x, double *B, int N, int M)
{
  VECTOR(y,N*N);

  iqprod(A,x,B, N,M, y );

  return(y);
}


/* 
   
   Cholesky decomposition: symmetric positive definite A = L*L'.

   Only the upper triangle of A needs to be provided.
   L is stored in the lower triangle of A.
   Main diagonal is stored in p.
   
*/

int
choldc(double *a, int N, double *p)
{
  int i,j,k, retVal = 0;
  double sum;

  for (i=0;i<N;i++) {
    for (j=i;j<N;j++) {

      sum = a[i+j*N];
      
      if (finite(sum))
	for (k=i-1;k>=0;k--)
	  sum -= a[i+k*N]*a[j+k*N];

      if (i==j) {

	if (sum<=0.0) {
	  /* fprintf(stderr,"Matrix is not positive definite %f.\n", sum); */
	  sum = 0;
	  retVal = -1;
	}
	
	p[i] = sqrt(sum);

      } else

	if (p[i] && finite(sum))
	  a[j+N*i] = sum/p[i];
	else
	  a[j+N*i] = 0;

    }
  }

#ifdef DEBUG

  for (i=0;i<N;i++) {
    for (j=0;j<N;j++)
      printf( "%f ", a[i+j*N] );
    printf("\n");
  }

  for (i=0;i<N;i++) 
    printf("%f ", p[i]);
  printf("\n");

#endif

  return retVal;
}

/* Inverse of lower triangular matrix L obtained from Cholesky decomp.
 Result is stored in the upper part of L with diagonal in p. */

void
invchol(double *a, double *p, int N)
{
  int i, j, k;
  double sum;

  for (i=0;i<N;i++) {
    if (p[i])
      p[i] = a[i+i*N] = 1./p[i];

    for (j=i+1;j<N;j++) {
      
      sum = -a[j+i*N]*p[i];
      
      if (finite(sum))
	for (k=i+1;k<j;k++)
	  sum -= a[j+k*N]*a[k+i*N];
      
      if (p[j] && finite(sum) )
	a[j+i*N] = sum/p[j];
      else
	a[j+i*N] = 0;

    }
  }

#ifdef DEBUG

  for (i=0;i<N;i++) {
    for (j=0;j<N;j++)
      printf( "%f ", a[i+j*N] );
    printf("\n");
  }

  for (i=0;i<N;i++) 
    printf("%f ", p[i]);
  printf("\n");

#endif

}

/* Product of L'*L (L - lower triangular with diagonal D) is stored in
   the upper triangle of L */
void
prodchol( double *L, double *D, int N )
{
  int i, j, k;

  for (i=0;i<N;i++) {
    for (j=i;j<N;j++) {
      if (D[j]) {
	L[i+j*N] = L[j+i*N]*L[j+j*N];
	for (k=j+1;k<N;k++)
	  if (D[k])
	    L[i+j*N] += L[k+i*N]*L[k+j*N];
      }
      else {
	L[i+j*N] = 0;
      }
      
    }
  }
}  


/* Pseudo inverse of a symmetric matrix */
int
sinv( double *A, int N, double *invA, double *detA )
{

  double *D, *colA, *colInvA;
  int i, j, k, retVal;

 if ( (D = calloc(N,sizeof(double))) == NULL )
    fprintf(stderr,"Cannot allocate sinv.");

  /* Copy upper triangle of A to invA */
  for (colA=A,colInvA=invA,i=0;i<N;i++,colA+=N,colInvA+=N)
    for (j=0;j<=i;j++)
      colInvA[j] = colA[j];

   
  /* Cholesky decomposition: A = invA*invA' */
  retVal = choldc(invA, N, D);
  
  /* Determinant */
  *detA = 1;
  for (i=0;i<N;i++)
    if ( D[i]>0 && finite(D[i]) )
      *detA *= (D[i]*D[i]);

  /* Cholesky inverse invA <- inv(invA) */
  invchol(invA, D, N);

  /* invA =  invA'*invA */
  prodchol( invA, D, N );
  
  /* Copy upper triangle of invA to its lower triangle */
  for (colA=invA+N,i=1;i<N;i++,colA+=N)
    for (colInvA=invA+i,j=0;j<i;j++,colInvA+=N)
      *colInvA = colA[j];
  
  
  free(D);
  
  return retVal;
}

double*
sinv_( double *A, int N )
{
  double detA;
  VECTOR(IA,N*N);

  if (A)
    sinv(A,N,IA,&detA);

  return(IA);
}


/* Lv = L'*v, L - lower triangular, p - diagonal elements of L */
void
lpvprod(double *L, double *p, double *v, int N, double *Lv)
{
  int i, j;
  double *Lcol;

  /* Product with main diagonal p */
  for (i=0; i<N; i++)
    Lv[i] = p[i]*v[i];

  /* Add products of off-diagonal elements */
  for (i=0, Lcol=L; i<N-1; i++, Lcol+=N) {
    for (j=i+1;j<N;j++)
      Lv[i] += Lcol[j]*v[j];
  }

}


/* d = v'*Av, A - symmetric NxN, non-negative definite, v - NXM */ 
int
mnorm(double *A, double *v, int N, int M, double *d)
{
  int i, j, retVal;
  double *p, *Lv;
  
  if ( (p = calloc(N,sizeof(double))) == NULL )
    fprintf(stderr,"Cannot allocate mdist.");
  if ( (Lv = calloc(N,sizeof(double))) == NULL )
    fprintf(stderr,"Cannot allocate mdist.");
  
  /* Factorize A = L*L' */
  retVal = choldc(A, N, p);
  
  for(i=0;i<M;i++,v+=N) {

    /* p = L'*v */
    lpvprod(A, p, v, N, Lv);
  
    /* d = p'*p */
    for(j=0,d[i]=0;j<N;j++)
      d[i] += Lv[j]*Lv[j];

    d[i] = sqrt( d[i] );
    
  }

  free(p);
  free(Lv);
  
  return retVal;
}

/*
  "Outter" quadratic operator:
  y <- y + alpha*A*x*A' + B
  A [NxM]
  x [MxM], symmetric
  y [NxN], symmetric
*/
void
oquadop( double alpha, double *A, double *x, double *B, int N, int M, int Mo,
	double *y )
{
  int i, j, k;

  /*
    Solve the problem by recursive calls to 2x2 case:
    A = [ a1 a2 ]
    x = [ x1 x2 ; x2' x3 ]
    A*x*A' = a1*x1*a1' + a1*x2*a2' + a2*x2'*x1' + a2*x3*a2'
    Start by selecting a1 to be the first column of A.
  */

  if (M>0) {

    /* a1 x1 a1' */
    for (i=0;i<N;i++)
      for (j=0;j<=i;j++) {

	if (x[0])
	  y[i+j*N] = y[j+i*N] += alpha*A[i]*A[j]*x[0];
	
	if (B)
	  y[i+j*N] = y[j+i*N] += B[i+j*N];
      }
  }
  
  if (M>1) {
    
    for (i=0;i<N;i++)
      for (j=0;j<=i;j++)
	
	/* a1 x2 a2' + a2 x2' a1' */
	for (k=1;k<M;k++)
	  
	  if (x[k])
	    y[i+j*N] = y[j+i*N] += 
	      alpha*( A[i]*A[j+N*k] + A[j]*A[i+k*N] )*x[k];
    
    /* a2 x3 a2' */
    oquadop( alpha, A+N, x+Mo+1, NULL, N, M-1, Mo, y );
    
  }
  
  
}


/*
  Linear operator:
  y <- y + alpha*A*x + B
  A [NxM]
  x [MxS]
  y [NxS]
*/
  void
  linop( double alpha, double *A, double *x, double *b, int N, int M, int S, 
       double *y )
{
  int i, j, k;

  for (j=0;j<S;j++) {

    for (i=0;i<N;i++,y++) {
      
      if (A)
	for (k=0;k<M;k++)
	  *y += alpha*A[i+N*k]*x[k];
      else
	*y = alpha*x[i];
      
      if (b)
	*y += b[i];
    }
    
    if (x)
      x += M;

    if (b)
      b += N;

  }

}

