function [ia,da] = sinv(a,err)
%
% sinv -- inverse of a symmetric non-negative definite matrix
%
% Usage
%   [ia,da] = sinv(a)
% Input:
%   a	- symmetric non-negative definite matrix
% Output:
%   ia	- inverse of 'a'
%   da  - determinant of 'a'
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:39:24 $

if (nargin<2)
  err = 0;
end

[r,p] = chol(a);
ir = inv(r);

da = prod(diag(r))^2;

if (p>0)
  % Not positive definite
  ia = zeros(size(a));
  ia(1:p-1,1:p-1) = ir*ir';
else
  ia = ir*ir';
end
