function sa = wsum(a,dim,w)
%
% WSUM -- weighted sum of elements
% 
% Usage:
%   s = wsum(a,dim,w)
% Input:
%   a		matrix
%   dim		dimension of summation (default 1)
%   w		summation weights (default 1)
% Output:
%   s		sum along dimension 'dim' weighted by weights in 'w'
%
% Example:
%   wsum([1 2 3;5 6 7],2,[0 1 0]) is [2;6]
%
% See also:
%   sum
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:39:24 $

if (nargin<2)
  dim = 1;
end

if (nargin<3)  
  
  sa = sum(a,dim);

else
  
  d = size(a);
  d(dim) = 1;
  perm = [ dim (1:dim-1) (dim+1:ndims(a)) ];
  
  sa = zeros(1,prod(d));
  a  = permute(a, perm);
    
  for k=1:length(w),
    sa = sa + w(k)*a(k,:);
  end
  
  sa = ipermute( reshape(sa,d(perm)), perm );
  
end

