function [C,R,y] = fa_init(y,N,mode,varargin)
%
% fa_init -- initialize factor anlizer parameters
%
% Usage:
%   [C,R] = fa_init(y,N,mode,...)
% Input:
%   y		observations
%   N		factor analyzer dimension
%   mode	initialization mode
%		fa_init(y,N,'orig',seed)
%		fa_init(y,N,'subspace')
% Output:
%   C		factor loading matrix
%   R		observation noise variance
%   

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:40 $

global FA_NL_C FA_NL_R

Mode = { 'orig' 'subspace' };
ModeString = sprintf('\t%s\n',Mode{:});

if (nargin<4)
  seed = 0;
end

if nargin<3,
  mode = Mode{1};
end

if (~iscell(y))
  y = {y};
end

S = length(y);
M = size(y{1},1);

cY = zeros(M);
Y  = [];

for s = 1:S,
  
  T = size(y{s},2);

  % Zero mean case
  my = mean(y{s},2);
  y{s}  = y{s} - my(:,ones(1,T));

  % List of observations
  Y = [ Y y{s} ];
  
  % Cumulative score
  cY = cY + y{s}*y{s}'/ T;
  
end

cY = cY / S;

switch( lower(mode) )
  
 case Mode(1)
  
  scale = det(cY)^(1/M);
  
  randn('seed',seed);
  
  if (isempty(FA_NL_C))
    C = randn(M,N)*sqrt(scale/N);
  else
    C = eye(M,N);
  end
  
  if (isempty(FA_NL_R))
    R = diag(diag(cY));
  else
    R = eye(M);
  end
  
 case Mode(2)
  
  [U,S,V] = svd(cY);
  
  C = U(:,1:N);
  R = diag( diag( ( eye(M) - C*pinv(C'*C)*C' ) * cY ));

 otherwise
  
  error( sprintf('Unknown mode. Supported modes are:\n %s', ModeString) );
  
end

