function [y,x] = fa_sim(m,Q,C,R, L,seed)
%
% fa_sim -- simulation of a FA
%
% Usage:
%   [y,x] = fa_sim(m,Q,C,R, L,seed)
% Input:
%   m		factor mean
%   Q		factor variance
%   C		loading matrix
%   R		observation noise variance
%   L		number of samples
%   seed	random generator seed (default 0)
% Output:
%   y		observations
%   x		factors
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:41 $

N = size(Q,1);
M = size(R,1);

if(nargin<6)
   seed = 0;
end

randn('seed',seed);

[Uq,DIAGq,UTq] = svd(Q);
[Ur,DIAGr,UTr] = svd(R);  

idx = ones(1,L);
x = m(:,idx) + Uq*sqrt(DIAGq)*randn(N,L);
y = C*x + Ur*sqrt(DIAGr)*randn(M,L);

