/*
 *   Copyright (c) 1999 by Vladimir Pavlovic
 *   $Id: fa_smt.c,v 2.0 2000/04/24 17:42:36 vladimir Exp vladimir $
 */

/* $Revision: 2.0 $ */
#include <math.h>
#include "mex.h"
#include "fa_util.h"

#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))

/* Input Arguments */

#define	Y	prhs[0]
#define X0	prhs[1]
#define Q	prhs[2]
#define C	prhs[3]
#define R	prhs[4]

/* Output Arguments */

#define COST	plhs[0]
#define EX	plhs[1]
#define	VX	plhs[2]	


void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double *pY, *pX0, *pQ, *pC, *pR;
  double *pCost, *pEx, *pVx;
  int N, M, T;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 5)  {
    mexErrMsgTxt("Requires at least five input arguments.");
  } else if (nlhs > 3) {
    mexErrMsgTxt("Requires three output arguments.");
  }
  
  /* Get dimsnesions */  
  M = mxGetM( C );
  N = mxGetN( C );
  T = mxGetN( Y );

  /* Get input pointers */
  pX0 = mxGetPr( X0 );
  pQ  = mxGetPr( Q );
  pC  = mxGetPr( C );
  pR  = mxGetPr( R );
  pY  = mxGetPr( Y );

  /* Create outputs */
  COST	= mxCreateDoubleMatrix(1,T, mxREAL);
  EX	= mxCreateDoubleMatrix(N,T, mxREAL);
  VX	= mxCreateDoubleMatrix(N,N, mxREAL);

  /* Get output pointers */
  pCost = mxGetPr( COST );
  pEx = mxGetPr( EX );
  pVx = mxGetPr( VX );

  if (nlhs>1)
    /* Inference */
    fa_pxy( pY, pC, pR, pX0, pQ, N, M, T,
	    pEx, pVx );
  
  /* Cost */
  fa_cost( pY, pC, pR, pX0, pQ, N, M, T, 
	   pCost );

  /* Clean up */
  fa_clean();

  return;
}

