function [cost,ex,vx] = fa_smt(y, m,Q,C,R)
%
% fa_smt -- factor analysis inference
%
%  Usage:
%    [cost,ex,vx] = fa_smt(y, m,Q,C,R)
%  Input:
%    y		observations
%    m		factor mean
%    Q		factor variance
%    C		factor loading matrix
%    R		observation noise variance
%  Output:
%    cost	-log{ P(Y) }
%    ex		E[x|y]
%    vx		Var[x|y]
%

%	$Revision: 1.3 $ $Date: 2000-03-03 14:12:47-05 $

L = size(y,2);
[M,N] = size(C);

idx = ones(1,L);


if (N>=M)
  [invO,detO] = sinv(C*Q*C'+R);
else
  invR = sinv(R);
  invO = invR*( eye(M) - C*sinv( C'*invR*C + sinv(Q) )*C'*invR );
  eigiO = eig(invO);
  detO = prod( 1./eigiO(find(eigiO>0)) );
end
beta = Q*C'*invO;

ey = C*m;
ex = m(:,idx) + beta*(y - ey(:,idx));
vx = Q - beta*C*Q;

dy = y-ey(:,idx);
cost =  ( sum( sum( (invO*dy).*dy ) )/L + log( detO ) + M*log(2*pi) ) / 2;
% cost =  ( trace( invO*dy*dy' )/L + log( detO ) + M*log(2*pi) ) / 2);


