#include "../emf/math_util.h"
#include "../mog/mog_util.h"

/* y = Ax + w */

void
fa_clean()
{
  emf_clean();
}

/* Pr( x | y ) */
void
fa_pxy( double *y,
	double *C, double *R, double *x0, double *Q, int N, int M, int T,
	double *ex, double *vx )
{
  
  static double *S = NULL;  static int dS = 0; 
  static double *B = NULL;  static int dB = 0;

  int t;

  if (M>dS) { S = realloc(S,M*M*sizeof(double)); dS = M; }
  if (M*N>dB) { B = realloc(B,M*N*sizeof(double)); dB = M*N; }

  /* Gain */
  /* S = C*R*C' + Q */
  /* B = Q*C'*inv(S) */
  oqprod( C,Q,R, M,N, S );
  prod( out_( Q,C, N,N,M ), sinv_( S, M ), N,M,M, B );
  
  /* ex = x0 + B*( y- C*x0 ) */
  for (t=0;t<T;t++)
    sum( prod_(B,diff_( y+t*M, prod_(C,x0,M,N,1), M), N,M,1), x0, N, ex+t*N );
  
  /* vx = (I-B*C)*Q  = Q - B*S*B' */
  if (vx)
    /* diff( Q, oqprod_( B,S, NULL, N,M ), N*N, vx ); */
    diff( Q, prod_(B, out_(C,Q,M,N,N), N,M,N), N*N, vx );
  
}

/* Pr( y ) */
void
fa_py( double *C, double *R, double *x0, double *Q, int N, int M,
       double *ey, double *vy )
{

  /* ey = C*x0 */
  prod( C,x0, M,N,1, ey );

  if (vy)
    /* vy = C*Q*C' + R */
    oqprod( C,Q,R, M,N, vy );

}

/* Cost = -log( P(Y) ) */
void
fa_cost( double *y,
	 double *C, double *R, double *x0, double *Q, int N, int M, int T,
	 double *cost )
{
  
  static double *ey = NULL, *vy = NULL;
  static int dim;
  double coef = 0;
  int t;
  
  if (M>dim) { 
    ey = realloc(ey,M*sizeof(double));
    vy = realloc(vy,M*M*sizeof(double));
    dim = M;
  }

  /* P(y) */
  fa_py( C,R,x0,Q, N,M, ey,vy );

  /* log P(Y) */
  mog_full( y, M, T, &coef, ey, vy, 1, 1, cost, NULL );

  for(t=0;t<T;t++)
    cost[t] = -cost[t];

}
