%
% Hidden Markov model
% ===================
%
% Model:
%
%   s_t ~ Pss(s_{t-1})
%   y_t ~ arbitrary distribution dependent on s_t
%
%   Initial condition:
%   s_0 ~ Ps0
%
% Notation:
%
%   s_t		hidden state at time 't'
%   y_t		observation (vector or symbol) at time 't'
%
%   N		# discrete hidden states
%   T		duration of observation sequence
%
%   Pss		state transition probability matrix
%		Pss(i,j) = Pr( s_t = i | s_{t-1} = j )
%		dimesiton: N x N
%   Ps0		initial state distribution vector
%		Ps0(i) = Pr( s_0 = i )
%		dimension: N x 1
%   Pys		probabilities of observed symbols or evaluated
%		pdfs of observed vectors for each hidden state
%		Pys(i,t) = Pr( y_t | s_t = i )
%		dimension: N x T
%
% Functions:
%
%   hmm_sim	simulation of a HMM state/observation sequence
%   hmm_fwd	forward state estimation
%   hmm_bwd	backward state estimation
%   hmm_smt	state smoothing (exact inference)
%   hmm_vite	state decoding (Viterbi inference)
%   hmm_infr	state inference
%   hmm_learn	model parameter learning
%   hmm_init	model parameter initialization
%
% Related toolkits:
%
%   mog		mixture of Gaussians toolkit
%   ghmm	Gaussian mixture hidden Markov model toolkit
%   dhmm	discrete observation HMM toolkit
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:50 $

