%
% Discrete observation HMM (DHMM)
% ==========================================
%
% Model:
%
%   s_t ~ Pss(s_{t-1})
%   y_t ~ Q(s_t)
%
%   Initial condition:
%   s_0 ~ Ps0
%
% Notation:
%
%   s_t		hidden state at time 't'
%   y_t		observation (vector or symbol) at time 't'
%
%   N		# discrete hidden states
%   M		# discrete observation states
%
%   Pss		state transition probability matrix
%		Pss(i,j) = Pr( s_t = i | s_{t-1} = j )
%		dimension: N x N
%   Ps0		initial state distribution vector
%		Ps0(i) = Pr( s_0 = i )
%		dimension: N x 1
%   Q		observation probabilities
%		Q(i,j) = Pr( y_t = i | s_t = j )
%		dimension: M x N
%   Y		sequence of observation vectors [ M x time ]
%
% Functions:
%
%   dhmm_sim	simulation of a DHMM state/observation sequence
%   dhmm_fwd	forward state estimation
%   dhmm_bwd	backward state estimation
%   dhmm_smt	state smoothing (exact inference)
%   dhmm_vite	state decoding (Viterbi inference)
%   dhmm_infr	state inference
%   dhmm_learn	model parameter learning
%   dhmm_init	model parameter initialization
%   dhmm_cat	combine several simple DHMMs into one complex DHMM
%
% Related toolkits:
%
%   hmm		hidden Markov model toolkit
%   vq		vector quantization toolkit
%   mc		Markov chains toolkit
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:52 $
