function beta = dhmm_bwd(y,Pss,Ps0,Q,scale,wS,wY)
%
% dhmm_bwd -- Discrete HMM backward state estimation 
%             (backward probability propagation)
%
% Usage:
%   beta = dhmm_bwd(y,Pss,Ps0,Q,scale,wS,wY)
%
% Input:
%   y		observation sequence [ 1 x time ]
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		observation probabilities  [ obs x state ]
%   scale	see 'dhmm_fwd'
%   wS		state transition probability weights (default 1)
%   wY		observation  probability weights (default 1)
%
% Output:
%   beta	Pr( s_t, observation sequence from time 't+1')
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   dhmm, dhmm_fwd, dhmm_vite, dhmm_smt, dhmm_infr
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:53 $

scale = exp(scale);

N = size(Pss,1);
T = size(y,2);

if (nargin<7)
  wS = ones(1,T);
end

if (nargin<8)
  wY = ones(1,T);
end

% Backward
beta(:,T) = ones(N,1)/scale(T);
for t=T-1:-1:1,
  Pys = ( Q(y(t+1),:)' ) .^ wY(t);
  beta(:,t) = (Pss.^w(t))'*(beta(:,t+1).*Pys)/scale(t);
end

beta = log(beta);
