function [P,Q,map,names] = dhmm_cat( Psyntax, modelList )
%
% dhmm_cat -- combine simple discrete HMMs into a complex HMM
%
% Usage:
%   [Pss,Q,map,names] = dhmm_cat( Psyntax, modelList )
%
% Input:
%   Psyntax	model transition probabilities
%   modelList	list of models & parameter
%
% Output:
%   Pss		complex model's state transition probability matrix
%   map		model state space mapping
%   names	list of simple model names contained in complex model
%
% Note:
%   (1) First state of each model is its entry state and last state is the
%       exit state.  Both states are 'non-visiting', i.e., expected durations
%       of those states are zero.  Hence, first columns of every Pss can
%       be viewed as initial state distributions and last columns are always
%       zero.
%   (2) Model transition probability Psyntax(i,j) is the probability that
%       the exit state of model 'j' transitions into the entry state of
%       model 'i'.
%   (3) Each element of model list can take one of two forms:
%		{ name, Pss, Q } or 
%		structure with 'Pss', 'Q', and 'name' fields.
%       Pss and Q are the simple model's parameters and 'name' is the
%       model's name.
%   (4) Mapping 'map' maps states of the complex model to simple models,
%       i.e., names{ map(2,j) } is the name of the simple model whose state
%       was mapped to state 'j' of the complex model.
%
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:53 $


P = []; pO = []; pI = []; map = []; cIdx = 0; names = {};
L = length(modelList);

% Structure or list flag
sFlag = isstruct( modelList{1} );

% Combine Markov chain parameters first
[P,map,names] = mc_cat( Psyntax, modelList );

% Combine observation matrices
Q = [];
for i=1:L,
  
  if sFlag,
    Q_i = modelList{i}.Q(:,2:end-1);
  else
    Q_i = modelList{i}{3}(:,2:end-1);
  end
  
  Q = cat(2,Q,Q_i);
  
end
