function [y,s] = dhmm_sim(Pss,Q,Ps0,T,seed)
%
% dhmm_sim -- simulate a discrete observation HMM
%
% Usage:
%   [y,s] = dhmm_sim(Pss,Q,Ps0,T,seed)
%
% Input:
%   Pss		state transition log probabilities [ to x from ]
%   Q		state onservation log matrix [obs x state]
%   Ps0		initial state log probabilities [ state x 1 ]
%   T		duration of simulation
%   seed	random generator seed
%
% Output:
%   y		observation sequences
%   s		hidden state sequence
%
% See also:
%   dhmm, hmm
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:55 $

if (nargin<5), seed = 0; end
if (nargin<6), mode = 'exp'; end
if (nargin<7), param = []; end

rand('seed',seed);
randn('seed',seed);

% Cumulative sum of Q
cQ   = cumsum(Q);

% Simulate Markov Chain
s = mc_sim(Pss,Ps0,T,seed);

for t=1:length(s),
  y(t) = sum( rand(1) > cQ(:,s(t)) ) + 1; 
end
