function [cost,es,cs] = dhmm_smt(y,Pss,Ps0,Q,wS,wY)
%
% dhmm_smt -- Discrete HMM state sequence smoothing 
%             (exact inference)
%
% Usage:
%   [cost,es,cs] = dhmm_smt(y,Pss,Ps0,Q,w)
% Input:
%   y		observation sequence [ 1 x time ]
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		observation probabilities  [ obs x state ]
%   wS		state transition probability weights (default 1)
%   wY		observation probability weights (default 1)
% Output:
%   cost	-log Pr( observation sequence )
%   es		E[ s_t | Y ] = Pr( s_t | observation sequence )
%   cs		E[ s_t s_{t-1}' | Y ] = 
%		  Pr( s_t s_{t-1} | observation sequence )
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   dhmm, dhmm_infr, dhmm_fwd, dhmm_bwd, dhmm_vite
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:55 $

T = size(y,2);
N = size(Pss,1);

if (nargin<6)
  wS = ones(1,T);
end

if (nargin<7)
  wY = ones(1,T);
end

% Forward...
[cost,es] = dhmm_fwd(y,Pss,Ps0,wS,wY);

% ...then smooth
if (nargout==2)
  es = hmm_f2s(Pss,es,wS);
elseif(nargout==3)
  [es,cs] = hmm_f2s(Pss,es,wS);
end


