/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:43:47 $
 */
 
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "hmm_util.h"

#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))

/* Input Arguments */

#define YDATA	prhs[0]
#define	PSS_IN	prhs[1]
#define PS0_IN	prhs[2]
#define QMAT_IN	prhs[3]
#define WS	prhs[4]
#define WY	prhs[5]
mxArray		*PSS;
mxArray		*PS0;
mxArray		*QMAT;

/* Output Arguments */

#define SCORE	plhs[0]
#define IDX	plhs[1]

static double *Q, *y, *wY;
static int D, M;

static double
discFun(int s, int t)
{
  if (wY)
    return wY[t]*Q[s*M+(int)y[t]-1];
  else
    return Q[s*M+(int)y[t]-1];    
}

void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Pss, *Ps0, *wS, *Score, *Idx;
  unsigned int	*tmpIdx = NULL;
  int		N, T, t, dims[2], *ir=NULL, *jc=NULL;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 4)  {
    mexErrMsgTxt("Requires at least four input arguments.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("Requires one or two output arguments.");
  }
  
  /* Duplicate parameters that need to be converted to log scale */
  PSS = mxDuplicateArray( PSS_IN );
  PS0 = mxDuplicateArray( PS0_IN );
  QMAT = mxDuplicateArray( QMAT_IN );

  N = mxGetM( PSS );
  M = mxGetM( QMAT );
  D = mxGetM( YDATA );
  T = mxGetN( YDATA );

  if ( mxGetN( PSS) != mxGetN( QMAT ) ) {
    mexErrMsgTxt("VITERBI Pss and Q dimensions do not agree.");
  }    


  /* Get input pointers */
  Pss = mxGetPr( PSS );
  if (mxIsSparse( PSS )) {
    ir  = mxGetIr( PSS );
    jc  = mxGetJc( PSS );
  }
  Ps0 = mxGetPr( PS0 );

  y = mxGetPr( YDATA );
  Q = mxGetPr( QMAT );
  
  if (nrhs<5)
    wS = NULL;
  else
    wS = mxGetPr( WS );

  if (nrhs<5)
    wY = NULL;
  else
    wY = mxGetPr( WY );

  /* Convert to log scale */
  lin2log( Pss, GET_TOTAL_ELEMENTS(PSS) );
  lin2log( Ps0, GET_TOTAL_ELEMENTS(PS0) );
  lin2log( Q, GET_TOTAL_ELEMENTS(QMAT) );

  /* Create outputs & assign pointers */
  SCORE = mxCreateDoubleMatrix(1, 1, mxREAL);
  Score = mxGetPr( SCORE );

  if (nlhs>1) {
    IDX = mxCreateDoubleMatrix(1, T, mxREAL);  /* Should use int array */
    Idx = mxGetPr( IDX );
    /* Create tmp index array */
    tmpIdx = mxCalloc(T,sizeof(unsigned int)); /* This shoud go away if IDX is int */
  }

  if (ir!=NULL && jc!=NULL)
    /* Call spares viterbi function */
    sparseViterbiFun(Pss, jc, ir, N, Ps0, discFun, wS, T, tmpIdx, Score);
  else
    viterbiFun(Pss, N, Ps0, discFun, wS, T, 0, tmpIdx, Score);

  if (nlhs>1) {
    /* Copy tmpIdx into Idx */
    for (t=0;t<T;t++) Idx[t] = tmpIdx[t];
    
    /* Free tmpIdx */
    mxFree(tmpIdx);
  }

  return;
}
