function [score,idx] = dhmm_vite(y,Pss,Ps0,Q,ws,wY)
%
% dhmm_vite --  Discrete HMM Viterbi state estimation
%
% Usage:
%   [score,idx] = dhmm_vite(y,Pss,Ps0,Q,wS,wY)
%
% Input:
%   y		observation sequence [ 1 x time ]
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		observation probabilities [ obs x state ]
%   wS		state transition probability weights (default 1)
%   wY		observation probability weights (default 1)
%
% Output:
%   score	-log Pr( y, decoded state sequence )
%   idx		decoded state sequence
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   dhmm, dhmm_smt, dhmm_infr
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:55 $

T = size(y,2);
N = size(Pss,1);

if (nargin<7)
  wS = ones(1,T);
end

if (nargin<8)
  wY = ones(1,T);
end

idx = zeros(1,T);
phi = zeros(N,T);

for t=1:T,
  if (t==1),
    delta0 = Ps0;
  else
    [delta0,phi(:,t)]  = max( Pss.^wS(t) .* delta0(:,ones(N,1))', [], 2 );
  end
  Pys = ( Q(y(t),:)' ) .^ wY(t);
  delta0 = delta0.*Pys;
end

% Backtracking
[score,idx(T)] = max(delta0);
for t=T-1:-1:1,
  idx(t) = phi(idx(t+1),t+1);
end

score = -max(log(delta0))/T;

