%
% Gaussian mixture observation HMM (GHMM)
% ==========================================
%
% Model:
%
%   s_t ~ Pss(s_{t-1})
%   c_t ~ Q(s_t)
%   y_t = C(c_t,s_t) + w(c_t,s_t)	E[w(c_t,s_t)] = 0
%					E[w(c_t,s_t)w(c_t,s_t)'] = R(c_t,s_t)
%
%   Initial condition:
%   s_0 ~ Ps0
%
% Notation:
%
%   s_t		hidden state at time 't'
%   y_t		observation (vector or symbol) at time 't'
%   c_t		mixture component at time 't'
%
%   N		# discrete hidden states
%   M		# mixture components per state
%   DIM		dimension of observation vector
%
%   Pss		state transition probability matrix
%		Pss(i,j) = Pr( s_t = i | s_{t-1} = j )
%		dimension: N x N
%   Ps0		initial state distribution vector
%		Ps0(i) = Pr( s_0 = i )
%		dimension: N x 1
%   Q		mixture weights
%		Q(i,j) = Pr( c_t = i | s_t = j )
%		dimension: M x N
%   C		mixture means
%		dimension: DIM x M x N
%   R		mixture covariances
%		dimension: diagonal covariance DIM x M x N
%		           full covariance DIM x DIM x M x N
%   Y		sequence of observation vectors [ DIM x time ]
%
% Functions:
%
%   ghmm_sim	simulation of a GHMM state/observation sequence
%   ghmm_fwd	forward state estimation
%   ghmm_bwd	backward state estimation
%   ghmm_smt	state smoothing (exact inference)
%   ghmm_vite	state decoding (Viterbi inference)
%   ghmm_infr	state inference
%   ghmm_learn	model parameter learning
%   ghmm_init	model parameter initialization
%   ghmm_comb	combine several simple GHMMs into one complex GHMM
%
% Related toolkits:
%
%   mog		mixture of Gaussians toolkit
%   hmm		hidden Markov model toolkit
%   vq		vector quantization toolkit
%   mc		Markov chains toolkit
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:56 $
