/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:43:47 $
 */
 
#include <math.h>
#include "mex.h"
#include "hmm_util.h"
#include "../emf/math_util.h"
#include "../mog/mog_util.h"

#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))

/* Input Arguments */

#define YDATA	prhs[0]
#define	PSS_IN	prhs[1]
#define PS0_IN	prhs[2]
#define QMAT_IN	prhs[3]
#define CMAT	prhs[4]
#define RMAT	prhs[5]
#define SCALE	prhs[6]
#define WS	prhs[7]
#define WY	prhs[8]
mxArray		*PSS;
mxArray		*PS0;
mxArray		*QMAT;

/* Output Arguments */

#define BETA	plhs[0]	

static double *Q, *C, *R, *y, *wY;
static int D, M, fullR;

static double
mogFun(int s, int t)
{
  double retVal;

  if (fullR)
    mog_full(y+t*D, D, 1, Q+M*s, C+s*M*D, R+s*M*D, M, 1, &retVal, NULL);
  else
    mog_diag(y+t*D, D, 1, Q+M*s, C+s*M*D, R+s*M*D, M, 1, &retVal, NULL);

  if (wY)
    retVal *= wY[t];

  return retVal;
}

void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Pss, *Ps0, *wS, *Beta, *Scale;
  int		N, T, t, dims[2], *ir=NULL, *jc=NULL;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 8)  {
    mexErrMsgTxt("GHMM_BWD requires at least seven input arguments.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("GHMM_BWD requires one output argument.");
  }
  
  
  /* Duplicate parameters that need to be converted to log scale */
  PSS = mxDuplicateArray( PSS_IN );
  PS0 = mxDuplicateArray( PS0_IN );
  QMAT = mxDuplicateArray( QMAT_IN );

  /* Get dimensions */
  N = mxGetM( PSS );
  M = mxGetM( QMAT );
  D = mxGetM( YDATA );
  T = mxGetN( YDATA );

  if ( mxGetN( PSS) != mxGetN( QMAT ) ) {
    mexErrMsgTxt("VITERBI Pss and Q dimensions do not agree.");
  }    

  /* Is R full? */
  if ( mxGetNumberOfDimensions(CMAT)>2 )
    fullR = mxGetNumberOfDimensions(RMAT) - mxGetNumberOfDimensions(CMAT);
  else
    fullR = GET_TOTAL_ELEMENTS(RMAT) - GET_TOTAL_ELEMENTS(CMAT); 

  /* Get input pointers */
  Pss = mxGetPr( PSS );
  if (mxIsSparse( PSS )) {
    ir  = mxGetIr( PSS );
    jc  = mxGetJc( PSS );
  }
  Ps0 = mxGetPr( PS0 );

  y = mxGetPr( YDATA );
  Q = mxGetPr( QMAT );
  C = mxGetPr( CMAT );
  R = mxGetPr( RMAT );
  
  Scale = mxGetPr( SCALE );

  if (nrhs<8)
    wS = NULL;
  else
    wS = mxGetPr( WS );

  if (nrhs<9)
    wY = NULL;
  else
    wY = mxGetPr( WY );

  /* Convert to log scale */
  lin2log( Pss, GET_TOTAL_ELEMENTS(PSS) );
  lin2log( Ps0, GET_TOTAL_ELEMENTS(PS0) );
  lin2log( Q, GET_TOTAL_ELEMENTS(QMAT) );

  /* Create outputs */
  BETA = mxCreateDoubleMatrix(N, T, mxREAL);

  /* Assign pointers to the various parameters */
  Beta = mxGetPr( BETA );


  /* Call forward function */
  if ( ir && jc )
    sparseBackwardFun(Pss, jc, ir, N, mogFun, T, wS, Scale, Beta);
  else
    backwardFun(Pss, N, mogFun, T, wS, Scale, 0, Beta);

  /* log -> lin */
  log2lin(Beta,N*T);

  return;
}
