function beta = ghmm_bwd(y,Pss,Ps0,Q,C,R,scale,wS,wY)
%
% hmm_bwd -- Gaussian mixture HMM backward state estimation 
%            (backward probability propagation)
%
% Usage:
%   beta = ghmm_bwd(y,Pss,Ps0,Q,C,R,scale,wS,wY)
%
% Input:
%   y		observation vector sequence [ dim x time ]
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		mixture weights [ mixture x state ]
%   C		observation mean [ dim x mixture x state ]
%   R		observation variance: diagonal [ dim x mixture x state ]
%		                      full [ dim x dim x mixture x state ]
%   scale	see 'ghmm_fwd'
%   wS		state transition probability weights (default 1)
%   wY		observation probability weights (default 1)
%
% Output:
%   beta	Pr( s_t, observation sequence from time 't+1')
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   ghmm, mog, ghmm_fwd, ghmm_vite, ghmm_smt, ghmm_infr
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:56 $

scale = exp(scale);

N = size(Pss,1);
T = size(y,2);

if (nargin<8)
  wS = ones(1,T);
end

if (nargin<9)
  wY = ones(1,T);
end

% Backward
beta(:,T) = ones(N,1)/scale(T);
for t=T-1:-1:1,
  Pys = exp( wY(t)*mog_eval(y(:,t+1),Q,C,R) );
  beta(:,t) = (Pss.^wS(t))'*(beta(:,t+1).*Pys)/scale(t);
end

beta = log(beta);
