function [P,Q,C,R,map,names] = ghmm_cat( Psyntax, modelList )
%
% ghmm_cat -- combine simple GHMMs into one complex GHMM
%
% Usage:
%   [Pss,Q,C,R,map,names] = ghmm_cat( Psyntax, modelList )
%
% Input:
%   Psyntax	model transition probabilities
%   modelList	list of models & parameter
%
% Output:
%   Pss		complex model's state transition probability matrix
%   Q		complex model's observation mixture weight matrix
%   C		complex model's observation means
%   R		complex model's observation variances
%   map		model state space mapping
%   names	list of simple model names contained in complex model
%
% See also:
%   'mc_cat', ghmm
%
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:56 $

L = length(modelList);

% Structure or list flag
sFlag = isstruct( modelList{1} );

% Diagonality of covariance matrix
if (sFlag)
  fFlag = ndims(modelList{1}.R) - ndims(modelList{1}.C);
else
  fFlag = ndims(modelList{1}{5}) - ndims(modelList{1}{4});
end  

% Combine Markov chain parameters first
[P,map,names] = mc_cat( Psyntax, modelList );

% Combine Gaussian mixture observation parameters
C = []; Q = []; R = [];

for m=1:L,
  
  if (sFlag)
    Qm = modelList{m}.Q;
    Cm = modelList{m}.C;
    Rm = modelList{m}.R;
  else
    Qm = modelList{m}{3};
    Cm = modelList{m}{4};
    Rm = modelList{m}{5};
  end
  
  % Combine Q's
  Q = cat(2,Q,Qm(:,2:end-1));
  
  % Combine C's
  if (isempty(C))
    C = Cm(:,:,2:end-1);
  else
    [N,M,S] = size(C);
    idxM = 1:size(Cm,2); 
    idxS = ( 1:size(Cm,3)-2 ) + S;
    C(:,idxM,idxS) = Cm(:,:,2:end-1);
  end
  
  % Combine R's
  if (isempty(R))
    if (fFlag)
      R = Rm(:,:,:,2:end-1);
    else
      R = Rm(:,:,2:end-1);
    end
  else
    if (fFlag)
      [N,N,M,S] = size(R);
      idxM = 1:size(Rm,3); idxS = ( 1:size(Rm,4)-2 ) + S;
      R(:,:,idxM,idxS) = Rm(:,:,:,2:end-1);
    else
      [N,M,S] = size(R);
      idxM = 1:size(Rm,2); idxS = ( 1:size(Rm,3)-2 ) + S;
      R(:,idxM,idxS) = Rm(:,:,2:end-1);
    end
  end
  
end
