/*
*	$Revision: 2.1 $ $Date: 2000/07/12 18:09:01 $
 */
 
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "hmm_util.h"
#include "../mog/mog_util.h"

#ifdef _WIN32
#define finite(x) _finite(x)
#endif

#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))


/* Input Arguments */

#define YDATA	prhs[0]
#define	PSS_IN	prhs[1]
#define PS0_IN	prhs[2]
#define QMAT_IN	prhs[3]
#define CMAT	prhs[4]
#define RMAT	prhs[5]
#define WS	prhs[6]
#define WY	prhs[7]
mxArray		*PSS;
mxArray		*PS0;
mxArray		*QMAT;

/* Output Arguments */

#define COST	plhs[0]
#define ES	plhs[1]
#define ESS	plhs[2]
#define	SCALE	plhs[3]	

static double *Q, *C, *R, *y, *wY;
static int D, M, fullR;

static double
mogFun(int s, int t)
{
  double retVal;

  if (fullR)
    mog_full(y+t*D, D, 1, Q+M*s, C+s*M*D, R+s*M*D, M, 1, &retVal, NULL);
  else
    mog_diag(y+t*D, D, 1, Q+M*s, C+s*M*D, R+s*M*D, M, 1, &retVal, NULL);
  
  if (wY)
    retVal *= wY[t];

  return retVal;
}

void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Pss, *Ps0, *wS, *Es, *Ess = NULL, *Cost, *Scale;
  int		i, N, T, t, nDim, dims[3], *ir = NULL, *jc = NULL;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 6)  {
    mexErrMsgTxt("Requires at least six input arguments.");
  } else if (nlhs > 4) {
    mexErrMsgTxt("Requires at most four output arguments.");
  }
  
  
  /* Duplicate parameters that need to be converted to log scale */
  PSS = mxDuplicateArray( PSS_IN );
  PS0 = mxDuplicateArray( PS0_IN );
  QMAT = mxDuplicateArray( QMAT_IN );

  /* Get dimensions */
  N = mxGetM( PSS );
  M = mxGetM( QMAT );
  D = mxGetM( YDATA );
  T = mxGetN( YDATA );

  if ( mxGetN( PSS) != mxGetN( QMAT ) ) {
    mexErrMsgTxt("Pss and Q dimensions do not agree.");
  }    

  /* Is R full? */
  if ( mxGetNumberOfDimensions(CMAT)>2 )
    fullR = mxGetNumberOfDimensions(RMAT) - mxGetNumberOfDimensions(CMAT);
  else
    fullR = GET_TOTAL_ELEMENTS(RMAT) - GET_TOTAL_ELEMENTS(CMAT); 

  /* Get input pointers */
  Pss = mxGetPr( PSS );
  if (mxIsSparse( PSS )) {
    ir  = mxGetIr( PSS );
    jc  = mxGetJc( PSS );
  }
  Ps0 = mxGetPr( PS0 );

  y = mxGetPr( YDATA );
  Q = mxGetPr( QMAT );
  C = mxGetPr( CMAT );
  R = mxGetPr( RMAT );
  
  if (nrhs<7)
    wS = NULL;
  else
    wS = mxGetPr( WS );

  if (nrhs<8)
    wY = NULL;
  else
    wY = mxGetPr( WY );

  /* Convert to log scale */
  lin2log( Pss, GET_TOTAL_ELEMENTS(PSS) );
  lin2log( Ps0, GET_TOTAL_ELEMENTS(PS0) );
  lin2log( Q, GET_TOTAL_ELEMENTS(QMAT) );

  /* Create outputs */
  COST = mxCreateDoubleMatrix(1,1, mxREAL);
  Cost = mxGetPr( COST );

  if (nlhs>1) {
    if ( (ES = mxCreateDoubleMatrix(N, T, mxREAL)) == NULL )
      mexErrMsgTxt("Cannot allocate ES memory.");
    Es = mxGetPr( ES );
  }

  if (nlhs>2) {
    nDim    = 3;
    dims[0] = N; dims[1] = N; dims[2] = T-1;
    if ( (ESS = mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL) ) == 
	 NULL )
      mexErrMsgTxt("Cannot allocate ESS memory.");      
    Ess = mxGetPr( ESS );
  }

  if (nlhs>3) {
    if ( (SCALE = mxCreateDoubleMatrix(1, T, mxREAL)) == NULL )
      mexErrMsgTxt("Cannot allocate SCALE memory.");
    Scale = mxGetPr( SCALE );
  } else 
    if ( (Scale = mxCalloc(T,sizeof(double)))==NULL)
      mexErrMsgTxt("Cannot allocate SCALE memory.");

  /* Call forward function */
  if ( ir && jc )
    sparseForwardFun(Pss, jc, ir, N, mogFun, T, Ps0, wS, Es, Scale, Ess);
  else
    forwardFun(Pss, N, mogFun, T, Ps0, wS, 0, Es, Scale, Ess);

  /* Cost */
  *Cost = cost(Scale,T);

  /* log -> lin */
  log2lin(Es,N*T);
  if (Ess) log2lin(Ess, N*N*(T-1));

  if (nlhs<3)
    mxFree(Scale);

  return;
}
