function [P,Q,C,R,p0] = ghmm_max( Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,Ts, ...
				  P,Q,C,R,p0 )
%
% ghmm_max -- maximization of GHMM parameters
%
% Usage:
%   [P,Q,C,R,p0] = ghmm_max(Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,Ts,P,Q,C,R,p0)
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:58 $

global VARTHD_GHMM_LEARN
if (~isempty(VARTHD_GHMM_LEARN))
   varThd = VARTHD_GHMM_LEARN;
else
   varThd = 1e-6;
end

N = size(Ess,1);
M = size(Ecs,1);
D = size(Eycs,1);
d = ndims(Eyycs) - ndims(Eycs);

P = zeros(N,N);
Q = zeros(M,N);
C = zeros(D,M,N);
if (d)
  R = zeros(D,D,M,N);
else
  R = zeros(D,M,N);
end

% Non-zero states
sIdx = find(Ess);

for c=1:M,
  for s=sIdx',
    
    Q(c,s)  = Ecs(c,s) / Ess(s);
    
    if (Q(c,s))
      
      C(:,c,s)  = Eycs(:,c,s) / Ecs(c,s);
      
      if (d),
	R(:,:,c,s) = ...
	    ( Eyycs(:,:,c,s) - C(:,c,s)*Eycs(:,c,s)' ) / ...
	    Ecs(c,s);
	% Impose minimal variance.
	[U,S,V] = svd(R(:,:,c,s));
	zIdx = find(S<varThd);
	S(zIdx) = varThd;
	R(zIdx,c,s) = U*S*V';
      else
	R(:,c,s) = ...
	    ( Eyycs(:,c,s) - C(:,c,s).*Eycs(:,c,s) ) / ...
	    Ecs(c,s);
	% Impose minimal variance
	zIdx = find(R(:,c,s)<varThd);
	R(zIdx,c,s) = varThd;
      end
      
    end
    
  end % for s
  
end % for c

p0 = Es0;

P(:,sIdx) = Dss(:,sIdx) ./ Ds(sIdx,ones(1,N))';

