function [cost,es,cs] = ghmm_smt(y,Pss,Ps0,Q,C,R,wS,wY)
%
% ghmm_smt -- mixture-of-Gaussian HMM state sequence smoothing 
%             (exact inference)
%
% Usage:
%   [cost,es,cs] = ghmm_smt(y,Pss,Ps0,Q,C,R,wS,wY)
% Input:
%   y		observation vector sequence [ dim x time ]
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		mixture weights [ mixture x state ]
%   C		observation mean [ dim x mixture x state ]
%   R		observation variance: diagonal [ dim x mixture x state ]
%		                      full [ dim x dim x mixture x state ]
%   wS		state transition probability weights (default 1)
%   wY		observation probability weights (default 1)
% Output:
%   cost	-log Pr( observation sequence )
%   es		E[ s_t | Y ] = Pr( s_t | observation sequence )
%   cs		E[ s_t s_{t-1}' | Y ] = 
%		  Pr( s_t s_{t-1} | observation sequence )
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   ghmm, mog, ghmm_infr, ghmm_fwd, ghmm_bwd, ghmm_vite
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:58 $

T = size(y,2);
N = size(Pss,1);

if (nargin<7)
  wS = ones(1,T);
end

if (nargin<8)
  wY = ones(1,T);
end

% Forward...
[cost,es] = ghmm_fwd(y,Pss,Ps0,Q,C,R,wS,wY);

% ...then smooth
if (nargout==2)
  es = hmm_f2s(Pss,es,wS);
elseif(nargout==3)
  [es,cs] = hmm_f2s(Pss,es,wS);
end

