function [Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,Ts] = ghmm_ucs(Y,es,cs,ec, ...
    Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,Ts,w)
%
% ghmm_ucs -- update Gaussian mixture HMM statistics
%
% Usage:
%   [Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,Ts] = 
%         ghmm_ucs( Y, es,cs,ec, Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds, Ts )
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:59 $

if nargin<13,
  w = 1;
end

% Update cumulative scores

T = size(Y,2);
N = size(es,1);
M = size(ec,1);
D = size(Y,1);
d = ndims(Eyycs) - ndims(Eycs);

% Update per-sequence parameters
for c=1:M,
  
  for s=1:N,
    
    Ecst = permute( ec(c,s,1:T), [2 3 1] ).*es(s,1:T);
        
    Ecs(c,s)    = Ecs(c,s) + w*sum( Ecst );
    Eycs(:,c,s) = Eycs(:,c,s) + w*Y*Ecst';
    
    if (d)
      % Full R
      Eyycs(:,:,c,s) = Eyycs(:,:,c,s) + w*Y*( Y.* ...
	  Ecst(ones(D,1),:) )';
    else
      % Diagonal R
      Eyycs(:,c,s) = Eyycs(:,c,s) + w*sum( (Y.^2)...
	  .* Ecst(ones(D,1),:), 2);
    end
    
  end
  
end

Ess = Ess + w*sum( es(:,1:T), 2);    % sum_{t=0}^{T-1} E[s_t]
Es0 = Es0 + w*es(:,1);               % E[s_0]

Dss = Dss + w*sum( cs, 3);
Ds  = Ds + w*sum( es(:,1:T-1), 2); % sum_{t=0}^{T-2} E[s_t]

Ts = Ts + w*T;

