function [score,idx] = ghmm_vite(y,Pss,Ps0,Q,C,R,wS,wY)
%
% ghmm_vite --  Gaussian mixture HMM Viterbi state estimation
%
% Usage:
%   [score,idx] = ghmm_vite(y,Pss,Ps0,Q,C,R,wS,wY)
%
% Input:
%   y		observation sequence [ dim x time ]
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		log mixture weights [ mixture x state ]
%   C		observation mean [ dim x mixture x state ]
%   R		observation variance: diagonal [ dim x mixture x state ]
%		                      full [ dim x dim x mixture x state ]
%   wS		state transition probability weights (default 1)
%   wY		observation probability weights (default 1)
%
% Output:
%   score	-log Pr( y, decoded state sequence )
%   idx		decoded state sequence
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   ghmm, mog,  ghmm_smt, ghmm_infr
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:59 $
  
T = size(y,2);
N = size(Pss,1);

if (nargin<7)
  wS = ones(1,T);
end

if (nargin<8)
  wY = ones(1,T);
end

idx = zeros(1,T);
phi = zeros(N,T);

for t=1:T,
  if (t==1),
    delta0 = Ps0;
  else
    [delta0,phi(:,t)]  = max( Pss.^wS(t) .* delta0(:,ones(N,1))', [], 2 );
  end
  Pys = exp( wY(t)*mog_eval(y(:,t),Q,C,R) );
  delta0 = delta0.*Pys;
end

% Backtracking
[score,idx(T)] = max(delta0);
for t=T-1:-1:1,
  idx(t) = phi(idx(t+1),t+1);
end

score = -max(log(delta0))/T;


