/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:43:48 $
 */
 
/* $Revision: 2.0 $ */
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "hmm_util.h"

#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))

/* Input Arguments */

#define	PSS_IN	prhs[0]
#define PYS	prhs[1]
#define SCALE	prhs[2]
#define W	prhs[3]
mxArray		*PSS;

/* Output Arguments */

#define BETA	plhs[0]


void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Pss, *Pys, *Beta, *w, *Scale;
  int		N, T;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 3)  {
    mexErrMsgTxt("BACKWARD requires at least three input arguments.");
  } else if (nlhs > 1) {
    mexErrMsgTxt("BACKWARD requires one output argument.");
  }
  
  
  /* Duplicate parameters that need to be converted to log scale */
  PSS = mxDuplicateArray( PSS_IN );

  /* Get dimensions */
  N = mxGetM( PSS );
  T = mxGetN( PYS );

  if ( mxGetN( PSS) != mxGetM( PYS ) ) {
    mexErrMsgTxt("BACKWARD Pss and Pys dimensions do not agree.");
  }    

  /* Get input pointers */
  Pss = mxGetPr( PSS );
  Pys = mxGetPr( PYS );
  Scale = mxGetPr( SCALE );
  if (nrhs==3)
    w = NULL;
  else
    w = mxGetPr( W );
  
  /* Convert to log scale */
  lin2log( Pss, GET_TOTAL_ELEMENTS(PSS) );

  /* Create outputs */
  BETA  = mxCreateDoubleMatrix(N, T, mxREAL);

  /* Assign pointers to the various parameters */
  Beta  = mxGetPr( BETA );

  /* Call backward function */
  backward(Pss, N, Pys, T, w, Scale, 0, Beta);

  /* log -> lin */
  log2lin(Beta,N*T);

  return;
}
