function beta = hmm_bwd(Pss,Pys,scale,w)
%
% hmm_bwd -- backward probability propagation
%
% Usage:
%   beta = hmm_bwd(Pss,Pys,scale,w)
% Input:
%   Pss		state transition probabilities [ to x from ]
%   Pys		evaluated observation log probabilities [ state x time ]
%   scale	scale factor [ 1 x time ] (see 'hmm_fwd')
%   w		state transition probability weights (default 1)
% Output:
%   beta	Pr( s_t, observation sequence from time 't+1')
%
% Note:
%   See notes for 'hmm_vite'.
% 
% See also:
%   hmm, hmm_fwd, hmm_smt, hmm_infr
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:59 $

% MEX file uses likelihoods, .m uses probabilities

Pys = exp(Pys);
scale = exp(scale);

N = size(Pss,1);
T = size(Pys,2);

if (nargin<4)
  w = ones(1,T);
end

% Backward
beta(:,T) = ones(N,1)/scale(T);
for t=T-1:-1:1,
  beta(:,t) = (Pss.^w(t))'*(beta(:,t+1).*Pys(:,t+1))/scale(t);
end

beta = log(beta);
