function varargout = hmm_disc(varargin)
%
% hmm_disc -- discrete distribution kernel
%
% Usage:
%   hmm_disc(Pys)	- set kernel parameters
%   x = hmm_disc(s)	- sample from 's'-th kernel
%   Pys = hmm_disc	- get kernel parameters
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:00 $

global HMM_DISC_PYS

if (nargin==1 & nargout==0)
  
  % Set parameters
  HMM_DISC_PYS = varargin{1};
  
elseif (nargin==1 & nargout==1)
  
  % Simulate observation
  T = size(varargin{1},2);
  
  if (ndims(HMM_DISC_PYS)>2)
    mode = 1; % observations on state transitions
  else
    mode = 0; % observations on states
  end

  r = rand(1,T);
  cPys = cumsum( HMM_DISC_PYS );
  
  if (mode==0)
    
    for t=1:T,
      
      s = varargin{1}(t);
      
      y(:,t) = sum( r(t) > cPys(:,s) ) + 1;
    
    end
    
  else
    
    s = varargin{1}(1);
    y(:,1) = sum( r(1) > cPys(:,1,s) ) + 1;
    
    for t=2:T,
      
      s1 = varargin{1}(t-1);
      s2 = varargin{1}(t);
      
      y(:,t) = sum( r(t) > cPys(:,s2,s1) ) + 1;
    
    end
    
  end
  
  varargout{1} = y;
  
else
  
  varargout{1} = HMM_DISC_PYS;

end
    
