/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:43:48 $
 */
 
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "hmm_util.h"

#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))

/* Input Arguments */

#define	PSS_IN	prhs[0]
#define ESF	prhs[1]
#define W	prhs[2]
mxArray		*PSS;

/* Output Arguments */

#define ES	plhs[0]
#define	ESS	plhs[1]	


void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Pss, *w, *EsF, *Es, *Ess;
  int		N, T, nDim, dims[3], *ir = NULL, *jc = NULL;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 2)  {
    mexErrMsgTxt("Requires at least two input arguments.");
  } else if (nlhs > 3) {
    mexErrMsgTxt("Requires two output arguments.");
  }
  
  /* Duplicate parameters that need to be converted to log scale */
  PSS = mxDuplicateArray( PSS_IN );

  /* Get dimensions */
  N = mxGetM( PSS );
  T = mxGetN( ESF );

  if ( N != mxGetM( ESF ) || N != mxGetN(PSS) ) {
    mexErrMsgTxt("Pss and Es dimensions do not agree.");
  }    

  /* Get input pointers */
  Pss = mxGetPr( PSS );
  if (mxIsSparse( PSS )) {
    ir  = mxGetIr( PSS );
    jc  = mxGetJc( PSS );
  }
  EsF = mxGetPr( ESF );
  if (nrhs<3)
    w = NULL;
  else
    w = mxGetPr( W );

  /* Convert to log scale */
  lin2log( Pss, GET_TOTAL_ELEMENTS(PSS) );

  /* Create output */
  if ( (ES = mxCreateDoubleMatrix(N, T, mxREAL)) == NULL )
    mexErrMsgTxt("SMOOTH cannot allocate ES memory.");
  Es = mxGetPr( ES );

  if (nlhs>1) {
    nDim    = 3;
    dims[0] = N; dims[1] = N; dims[2] = T-1;
    if ( (ESS = mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL) ) == 
	 NULL )
      mexErrMsgTxt("SMOOTH cannot allocate ESS memory.");      
    Ess = mxGetPr( ESS );
  } else
    Ess = NULL;

  /* lin -> log */
  lin2log(EsF,N*T);

  /* Call smooth function */
  if ( ir && jc )
    sparseSmooth2( EsF, Pss, jc, ir, w, N, T, 0, Es, Ess );
  else
    smooth2( EsF, Pss, w, N, T, 0, Es, Ess );

  /* log -> lin */
  log2lin(Es,N*T);
  if (Ess) log2lin(Ess,N*N*(T-1));

  return;
}
