function [Es,Cs] = hmm_f2s(Pss,es,w)
%
% hmm_f2s -- HMM smoothing
%
% Usage:
%   [Es,Cs] = hmm_f2s(Pss,es,w)
% Input:
%   Pss		state transition log probabilities [ to x from ]
%   es		forward state estimates, E[s_t | Y_t]
%   w		state transition probability weights (default 1)
% Output:
%   Es		smoothed state estimates, E[s_t | Y_T]
%   Cs		smoothed cross-correlation estimates, E[s_t s_{t-1}' | Y_T]
%   
% See also:
%   hmm_fwd, hmm, hmm_infr, hmm_bwd, hmm_vite, hmm_sim

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:00 $

T = size(es,2);
N = size(Pss,1);
i = ones(1,N);

if (nargin<3)
  w = ones(1,T);
end

Es(:,T) = es(:,T);

for t=T-1:-1:1,
  
  % Predict from forward estimate
  esp = Pss*es(:,t);
  
  % Pr(s_{t+1}=i, s_t=j | Y_T) = 
  %  Pr(s_{t+1}=i | s_t=j) Pr(s_t=j|Y_t) Pr(s_{t+1}=i|Y_T) / Pr(s_{t+1}=i|Y_t)
  cs = zeros(N,N);
  nz = find(esp);
  cs(nz,:) = ( Pss(nz,:).^w(t) ) .* es(:,t*i(nz))' .* Es(nz,(t+1)*i) ...
      ./ esp(nz,i);
  
  % Pr(s_t|Y_T) = sum_{s_{t+1}} Pr(s_{t+1}, s_t | Y_T)
  Es(:,t) = sum(cs,1)';
  
  if (nargout>2)
    Cs(:,:,t) = cs;
  end
  
end
