/*
*	$Revision: 2.1 $ $Date: 2000/07/12 18:09:38 $
 */
 
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "hmm_util.h"

#ifdef _WIN32
#define finite(x) _finite(x)
#endif


#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))

/* Input Arguments */

#define	PSS_IN	prhs[0]
#define PYS	prhs[1]
#define PS0_IN	prhs[2]
#define W	prhs[3]
mxArray		*PSS;
mxArray		*PS0;

/* Output Arguments */

#define COST	plhs[0]
#define ES	plhs[1]
#define ESS	plhs[2]
#define	SCALE	plhs[3]	

void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Pss, *Pys, *Ps0, *w, *Es, *Ess = NULL, *Cost, *Scale;
  int		i, j, N, T, *ir = NULL, *jc = NULL, nDim, dims[3];
  
  /* Check for proper number of arguments */
  
  if (nrhs < 3)  {
    mexErrMsgTxt("Requires at least three input arguments.");
  } else if (nlhs > 4) {
    mexErrMsgTxt("Requires at most four output arguments.");
  }
  
  /* Duplicate parameters that need to be converted to log scale */
  PSS = mxDuplicateArray( PSS_IN );
  PS0 = mxDuplicateArray( PS0_IN );

  /* Get dimsnesions */
  N = mxGetM( PSS );
  T = mxGetN( PYS );

  if ( mxGetN( PSS) != mxGetM( PYS ) ) {
    mexErrMsgTxt("Pss and Pys dimensions do not agree.");
  }    

  /* Get input pointers */
  Pss = mxGetPr( PSS );
  if (mxIsSparse( PSS )) {
    ir  = mxGetIr( PSS );
    jc  = mxGetJc( PSS );
  }
  Pys = mxGetPr( PYS );
  Ps0 = mxGetPr( PS0 );
  if (nrhs==3)
    w = NULL;
  else
    w = mxGetPr( W );

  /* Convert to log scale */
  lin2log( Pss, GET_TOTAL_ELEMENTS(PSS) );
  lin2log( Ps0, GET_TOTAL_ELEMENTS(PS0) );

  /* Create outputs */
  COST = mxCreateDoubleMatrix(1,1, mxREAL);
  Cost = mxGetPr( COST );

  if (nlhs>1) {
    if ( (ES = mxCreateDoubleMatrix(N, T, mxREAL)) == NULL )
      mexErrMsgTxt("Cannot allocate ES memory.");
    Es = mxGetPr( ES );
  }

  if (nlhs>2) {
    nDim    = 3;
    dims[0] = N; dims[1] = N; dims[2] = T-1;
    if ( (ESS = mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL) ) == 
	 NULL )
      mexErrMsgTxt("Cannot allocate ESS memory.");      
    Ess = mxGetPr( ESS );
  }

  if (nlhs>3) {
    if ( (SCALE = mxCreateDoubleMatrix(1, T, mxREAL)) == NULL )
      mexErrMsgTxt("Cannot allocate SCALE memory.");
    Scale = mxGetPr( SCALE );
  } else 
    if ( (Scale = mxCalloc(T,sizeof(double)))==NULL)
      mexErrMsgTxt("Cannot allocate SCALE memory.");

  /* Call forward function */
  if ( ir && jc )
    sparseForward(Pss, jc, ir, N, Pys, T, Ps0, w, Es, Scale, Ess);
  else
    forward(Pss, N, Pys, T, Ps0, w, 0, Es, Scale, Ess);

  /* Cost */
  *Cost = cost(Scale,T);

  /* log -> lin */
  log2lin(Es,N*T);
  if (Ess) log2lin(Ess, N*N*(T-1));

  if (nlhs<3)
    mxFree(Scale);

  return;
}
