function [gPss,gPs0,varargout] = hmm_grad(y,Pss,Ps0,varargin)

T = size(y,2);
N = size(Pss,1);

if (nargin==4)
  % %%%%%%%%%%%%%%%%%%%%%
  % Discrete observations
  % %%%%%%%%%%%%%%%%%%%%%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:01 $
  
  Pys = varargin{1};
  
  % Inference
  [cost,es,cs] = hmm_smt(Pss,Pys'*y,Ps0);

  % Observation
  gPys = y*es' ./ exp(Pys) / T;
  gPys = gPys - ones(M,1)*sum(gPys,1)/M;	% projection
  
  varargout{1} = gPys;
  
else
  % %%%%%%%%%%%%%%%%%%%%%
  % Gaussian observations
  % %%%%%%%%%%%%%%%%%%%%%

  Q = varargin{1}; C = varargin{2}; R = varargin{3};
  
  M = size(Q,1);
  Na = size(C,1);
  
  % Inference
  [Pys,Pycs] = mog_eval(y,Q,C,R);
  [cost,es,cs] = hmm_smt(Pss,Pys,Ps0);
  
  tmp = permute( Pys, [ 3 1 2 ]);
  ec = exp( Pycs - tmp(ones(M,1),:,:) );
    
  for c=1:M,
    for s=1:N,
      
      Ecst	   = permute( ec(c,s,1:T), [2 3 1] ) .* es(s,1:T);
      Ecs(c,s)	   = sum( Ecst );
      Eycs(:,c,s)  = y*Ecst';
      Eyycs(:,c,s) = sum( (y.^2) .* Ecst(ones(Na,1),:), 2);
    
    end
  end

  % Mixture coefs
  gQ = Ecs ./ exp(Q) / T;
  gQ = gQ - ones(M,1)*sum(gQ,1)/M;	% projection

  for s=1:N,
    for c=1:M,
      
      % Mean
      gC(:,c,s) = diag(1./R(:,c,s)) * ( Eycs(:,c,s) - C(:,c,s)*Ecs(c,s) ) ...
	  / T;
      
      % Inverse variance
      gRI(:,c,s) = ( - ( Eyycs(:,c,s) - 2*C(:,c,s).*Eycs(:,c,s) ...
	  + C(:,c,s).^2.*Ecs(c,s) ) + R(:,c,s)*Ecs(c,s) ) / (2*T);
      
    end
  end
  
  varargout{1} = gQ;
  varargout{2} = gC;
  varargout{3} = gRI;
  
end

% Transition
gPss = sum(cs,3) ./ exp(Pss) / (T-1);
gPss = gPss - ones(N,1)*sum(gPss,1)/N;	% projection


% Initial
gPs0 = es(:,1) ./ exp(Ps0);
gPs0 = gPs0 - sum(gPs0)/N;		% projection
