function varargout = hmm_mog(varargin)
%
% hmm_mog -- mixture of Gaussians distribution kernel
%
% Usage:
%   hmm_mog(Q,C,R)	- set kernel parameters
%   x = hmm_mog(s)	- sample from 's'-th parameter set
%   [Q,C,R] = hmm_mog	- get kernel parameters
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:02 $

global HMM_MOG_Q HMM_MOG_C HMM_MOG_R

if (nargin==3)
  
  % Set parameters
  HMM_MOG_Q = varargin{1};
  HMM_MOG_C = varargin{2};
  HMM_MOG_R = varargin{3};
  
elseif (nargin==1)
  
  % Simulate observation
  [M N] = size(HMM_MOG_C);
  T = size(varargin{1},2);
  
  r1 = rand(1,T);
  r2 = randn(M,T);
  
  % Cumulative sum of HMM_MOG_Q
  cQ = cumsum(HMM_MOG_Q);
  
  for t=1:T,
   
    s = varargin{1}(t);

    c = sum( r1(t) > cQ(:,s) ) + 1;
  
    % So far only on diagonal variance
    x(:,t) = HMM_MOG_C(:,c,s) + sqrt( HMM_MOG_R(:,c,s) ).* r2(:,t);
    
  end
  
  varargout{1} = x;
  
else
  
  varargout{1} = HMM_MOG_Q;
  varargout{2} = HMM_MOG_C;
  varargout{3} = HMM_MOG_R;

end
    
