function [y,s] = hmm_sim(Pss,Ps0,kernel,T,seed)
%
% hmm_sim -- simulate an HMM
%
% Usage:
%   [y,s,c] = hmm_sim(Pss,Ps0,kernel,T,seed)
%
% Input:
%   Pss		state transition log probabilities [ to x from ]
%   Ps0		initial state log probabilities [ state x 1 ]
%   kernel	observation kernel
%   T		duration of simulation
%   seed	random generator seed
%
% Output:
%   y		observation sequences
%   s		hidden state sequence
%
% See also:
%   hmm
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:02 $

if (nargin<5), seed = 0; end

rand('seed',seed);
randn('seed',seed);

r = rand(1,T);

% Cumulative sum of Pss
cPss = cumsum(Pss);

% Simulate Markov Chain
cPs0 = cumsum(Ps0);
s(1) = sum(r(1) > cPs0)+1;

for t=2:T,
  s(t) = sum( r(t) > cPss(:,s(t-1)) )+1;
end

% Simulate Observations
y = feval(kernel,s);
