/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:43:49 $
 */
 
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "hmm_util.h"

#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))

/* Input Arguments */

#define	PSS_IN	prhs[0]
#define PYS	prhs[1]
#define PS0_IN	prhs[2]
#define W	prhs[3]
mxArray		*PSS;
mxArray		*PS0;

/* Output Arguments */

#define COST	plhs[0]
#define ES	plhs[1]
#define	ESS	plhs[2]	


void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Pss, *Pys, *Ps0, *w, *Es, *Ess = NULL, *Cost;
  int		N, T, nDim, dims[3], *ir = NULL, *jc = NULL;
  double	*EsF, *Scale;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 3)  {
    mexErrMsgTxt("Requires at least three input arguments.");
  } else if (nlhs > 3) {
    mexErrMsgTxt("Requires two output arguments.");
  }
  
  /* Duplicate parameters that need to be converted to log scale */
  PSS = mxDuplicateArray( PSS_IN );
  PS0 = mxDuplicateArray( PS0_IN );

  /* Get dimsnesions */  
  N = mxGetM( PSS );
  T = mxGetN( PYS );

  if ( mxGetN( PSS) != mxGetM( PYS ) ) {
    mexErrMsgTxt("Pss and Pys dimensions do not agree.");
  }    

  /* Get input pointers */
  Pss = mxGetPr( PSS );
  if (mxIsSparse( PSS )) {
    ir  = mxGetIr( PSS );
    jc  = mxGetJc( PSS );
  }
  Pys = mxGetPr( PYS );
  Ps0 = mxGetPr( PS0 );
  if (nrhs==3)
    w = NULL;
  else
    w = mxGetPr( W );

  /* Convert to log scale */
  lin2log( Pss, GET_TOTAL_ELEMENTS(PSS) );
  lin2log( Ps0, GET_TOTAL_ELEMENTS(PS0) );

  /* Create outputs */
  COST	= mxCreateDoubleMatrix(1,1, mxREAL);
  Cost = mxGetPr( COST );

  if (nlhs>1) {
    if ( (ES = mxCreateDoubleMatrix(N, T, mxREAL)) == NULL )
      mexErrMsgTxt("Cannot allocate ES memory.");
    Es = mxGetPr( ES );
  }

  if (nlhs>2) {
    nDim    = 3;
    dims[0] = N; dims[1] = N; dims[2] = T-1;
    if ( (ESS = mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL) ) == 
	 NULL )
      mexErrMsgTxt("Cannot allocate ESS memory.");      
    Ess = mxGetPr( ESS );
  }

  /* Allocate tmp data storage (esF & Scale) */
  if ( (EsF = mxCalloc(N*T,sizeof(double))) == NULL )
    mexErrMsgTxt("Cannot allocate ESF memory.");
  if ( (Scale = mxCalloc(T,sizeof(double))) == NULL )
    mexErrMsgTxt("Cannot allocate SCALE memory.");
    
  /* Forward first... */
  if ( ir && jc )
    sparseForward(Pss, jc, ir, N, Pys, T, Ps0, w, EsF, Scale, NULL);
  else
    forward(Pss, N, Pys, T, Ps0, w, 0, EsF, Scale, NULL);

  /* ...then smoothing */
  if (nlhs>1) {

    if ( ir && jc )
      sparseSmooth2( EsF, Pss, jc, ir, w, N, T, 0, Es, Ess );
    else
      smooth2( EsF, Pss, w, N, T, 0, Es, Ess );
 
    /* log -> lin */
    log2lin(Es,N*T);
    if (Ess) log2lin(Ess, N*N*(T-1));

  }

  *Cost = cost(Scale,T);

  /* Free tmp data storage */
  mxFree(Scale);
  mxFree(EsF);

  return;
}
