function [cost,es,cs] = hmm_smt(Pss,Pys,Ps0,w)
%
% hmm_smt -- HMM state sequence smoothing (exact inference)
%
% Usage:
%   [cost,es,cs] = hmm_smt(Pss,Pys,Ps0,w)
% Input:
%   Pss		state transition probabilities [ to x from ]
%   Pys		evaluated observation log probabilities [ state x time ]
%   Ps0		initial state probabilities [ state x 1 ]
%   w		state transition probability weights (default 1)
% Output:
%   cost	-log Pr( observation sequence )
%   es		E[ s_t | Y ] = Pr( s_t | observation sequence )
%   cs		E[ s_t s_{t-1}' | Y ] = 
%		  Pr( s_t s_{t-1} | observation sequence )
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   hmm, hmm_infr, hmm_fwd, hmm_bwd, hmm_vite, hmm_sim
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:02 $

T = size(Pys,2);
N = size(Pss,1);

if (nargin<4)
  w = ones(1,T);
end

% Forward...
[cost,es] = hmm_fwd(Pss,Pys,Ps0,w);

% ...then smooth
if (nargout==2)
  es = hmm_f2s(Pss,es,w);
elseif(nargout==3)
  [es,cs] = hmm_f2s(Pss,es,w);
end



