/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:43:49 $
 */
 
#include "hmm_util.h"
#include "../emf/math_util.h"

#define log_2pi	1.837877066


#define CHECK_FINITE
#define INDEX_OFFSET 1

void
hmm_clean()
{
  emf_clean();
}

void
forward( double *Pss, int N, double *Pys, int T, double *Ps0, double *wS,
	 int mode, 
	 double *Es, double *Scale, double *Ess )
{
  double	*EsOld;
  double	ws, Essmn;
  int		n, m, t;
  
  /* Initialize: t=0 */
  if (wS)
    ws = wS[0];
  else
    ws = 1;
  Scale[0] = -HUGE;
  
  /* Observation + prior */
  for (m=0; m<N; m++)
    if ( finite(Pys[m]) && finite(Ps0[m]) ) {
      Es[m] = ws*Pys[m] + Ps0[m];
      Scale[0] = logSum( Scale[0], Es[m] );
    } else
      Es[m] = -HUGE;

  /* Rescale */
  for (m=0; m<N; m++)
    if ( finite(Es[m]) && finite(Scale[0]) )
      Es[m] -= Scale[0];

  /* Update pointers */
  Es    += N;
  Pys   += N;
  
  /* For time steps 1...T-1 */
  for (t=1;t<T;t++) {
    
    Scale[t]	= -HUGE;
    EsOld	= Es - N;
    
    /* Transition & observation weights */
    if (wS) 
      ws = wS[t];
    else
      ws = 1;
    
    /* For each TO state */
    for (m=0; m<N; m++) {
      
      /* Propagation */
      Es[m] = -HUGE;
      
      if ( finite(Pys[m]) ) {
	
	/* For each FROM state */
	for (n=0; n<N; n++) {
	  
	  if ( finite(Pss[n*N+m]) && finite(EsOld[n]) )
	    Essmn = EsOld[n] + ws*Pss[n*N+m] + Pys[m];
	  else
	    Essmn = -HUGE;

	  Es[m] = logSum( Es[m], Essmn );
	  
	  if (Ess)
	    Ess[m+n*N] = Essmn;
	  
	} /* n=0...N-1 */
		
	/* Scale */
	Scale[t] = logSum( Scale[t], Es[m] );

      } else {

	if (Ess)
	  for (n=0; n<N; n++)
	    Ess[m+n*N] = -HUGE;
	
      }
      
    } /* m=0...N-1 */
    
    /* Rescale */
    for (m=0; m<N; m++) {
      
      if ( finite(Es[m]) && finite(Scale[t]) ) {
	Es[m] -= Scale[t];
	if (Ess)
	  for (n=0; n<N; n++)
	    Ess[m+n*N] -= Scale[t];
      }
      
    }
    
    /* Update pointers */
    Es    += N;
    Pys   += N;
    if (Ess)
      Ess += N*N;
    if (mode)
      Pss += N*N;

  }
  
}

void
backward( double *Pss, int N, double *Pys, int T, double *wS, double *Scale, 
	  int mode,
	  double *Beta )
{
  double	*BetaOld, *PysOld, ws;
  int		n, m, t;
  
  /* Point to the end of arrays */
  Beta  += N*(T-1);
  PysOld  = Pys + N*T;
  BetaOld = Beta + N;
  if (mode)
    Pss += N*N*(T-2);
  
  /* For each time step */
  for (t=T-1; t>=0; t--) {

    /* Transition weight */
    if (wS && t<T-1)
      ws = wS[t+1];
    else
      ws = 1;

    /* For each FROM state */
    for (m=0; m<N; m++) {
      
      /* Propagation */
      
      if (t<T-1) {
	
	Beta[m] = -HUGE;
	
	/* For each TO state */
	for (n=0; n<N; n++) {
	  if ( 
#ifdef CHECK_FINITE
	      finite(Pss[m*N+n]) && finite(BetaOld[n]) && finite(PysOld[n])
#else
	      1
#endif
	      )
	      Beta[m] = logSum( Beta[m], BetaOld[n] + ws*Pss[m*N+n] 
				+ PysOld[n]);
	}
	
	if ( 
#ifdef CHECK_FINITE
	    finite( Beta[m] ) && finite( Scale[t] )
#else
	    1
#endif
	    )
	  Beta[m] -= Scale[t];
	else
	  Beta[m] = -HUGE;
	
      } else {

	Beta[m] = -Scale[t];
	
      }
      
      
    }
    
    Beta    -= N;
    BetaOld -= N;
    PysOld  -= N;
    if (t<T-1 && mode)
      Pss -= N*N;
    
    
  }
  
}

void
viterbiForwardStep( double *Pss, int N, double *Pys, double *Pin, double *wS,
		    double *Pout, int *idx )
{
  int n, m;
  double score, ws;

  /* For each to state */
  for (m=0;m<N;m++) {

    /* Transition weight */
    if (wS)
      ws = wS[0];
    else
      ws = 1;

    Pout[m] = -HUGE;
    idx[m]  = 0;

    /* For each from state */
    for (n=0;n<N;n++) {
      
      if ( finite(Pss[m+N*n]) && finite(Pin[n]) ) {

	score = Pin[n] + ws*Pss[m+N*n];

	if (score > Pout[m]) {
	  Pout[m] = score;
	  idx[m]  = n;
	}
      }
    }

    if (finite(Pys[m]) && finite(Pout[m]))
      Pout[m] += Pys[m];
    else
      Pout[m] = -HUGE;
    
    
  }
  
}


void
viterbi( double *Pss, int N, double *Pys, int T, double *Ps0, double *wS,
	 int mode,
	 unsigned int *Idx, double *score )
{
  double	*deltaNew, *deltaOld, *Delta, *pss, *tmp, ws;
  unsigned int	n, m, *Phi, s;
  int		t;

  if (Idx) {
    Phi   = calloc(N*T,sizeof(unsigned int));
  }
  Delta = calloc(2*N,sizeof(double));

  deltaNew = Delta;
  deltaOld = Delta+N;

  /* For each time step */
  for (t=0;t<T;t++, Pys+=N) {

     /* Transition weight */
    if (wS)
      ws = wS[t];
    else
      ws = 1;
   

    /* For each TO state */
    for (m=0; m<N; m++) {

      /* Propagation */

      if (t>0) {
	
	pss = Pss + m;
	deltaNew[m] = -HUGE;
	
	/* For each FROM state */
	for (n=0; n<N; n++, pss+=N) {
	  if ( finite(*pss) && finite(deltaOld[n]) ) {
	    
	    score[0] = deltaOld[n] + (*pss)*ws;
		    
	    if (score[0] > deltaNew[m]) {
	      deltaNew[m] = score[0];
	      if (Idx)
		Phi[m]      = n;
	    }
	  }
	}
	
      } else {
	
	deltaNew[m] = *(Ps0++)*ws;
      
      }

      /* Observation */
      if ( finite(Pys[m]) && finite(deltaNew[m]) )
	deltaNew[m] += Pys[m];
      else
	deltaNew[m] = -HUGE;


    }


    tmp = deltaOld;
    deltaOld = deltaNew;
    deltaNew = tmp;

    if(t>0 && mode)
      Pss += N*N;

    if (Idx)
      Phi+=N;

  }

  /* Backtracing */

  score[0] = -HUGE;
  for (m=0;m<N;m++) {
    if (deltaOld[m] > score[0]) {
      score[0] = deltaOld[m];
      if (Idx)
	Idx[T-1] = m + INDEX_OFFSET;
    }
  }

  if (Idx) {

    Phi-=N;
    
    for(t=T-2;t>=0;t--,Phi-=N) {
      Idx[t] = Phi[ Idx[t+1] - INDEX_OFFSET ] + INDEX_OFFSET;
    }
    
  }
  
  /* This is for compatibility only */
  score[0] *= -1.0/T;

  free(Delta);
  if (Idx)
    free(Phi);
      
}


double
cost(double *Scale, int T)
{
  int t;
  double retVal = 0.0;

  for (t=0;t<T;t++,Scale++)
    retVal += *Scale;

  return -retVal/T;
}

void
smooth0( double *EsF, double *Beta, double *Scale, int N, int T,
	 double *Es )
{
  int t, n;
  double logVal;

  for (t=0;t<T;t++,Scale++)
    for (n=0;n<N;n++, EsF++, Beta++, Es++) {
      logVal = *EsF + *Beta + *Scale;
      if ( finite(logVal) )
	*Es = exp( logVal );
      else
	*Es = 0.;
    }
}


void
smooth1( double *Es, double *Beta, double *Pss, double *Pys, double *wS,
	 int N, int T, int mode, 
	 double *Cs )
{
  int t, n, m;
  double logVal, ws;
  
  /* One time step difference */
  Pys += N;
  Beta += N;

  for (t=0;t<T-1;t++) {
    
     /* Transition weight */
    if (wS)
      ws = wS[t+1];
    else
      ws = 1;
    

    for (n=0;n<N;n++)
      
      for (m=0;m<N;m++) {

	logVal = Es[n] + ws*Pss[n*N+m] + Beta[m] + Pys[m];
	
	if ( finite( logVal ) )
	  Cs[n*N+m] = exp( logVal );
	else
	  Cs[n*N+m] = 0;
      
      }

    Cs    += N*N;
    Es    += N;
    Beta  += N;
    Pys   += N;

    if(mode)
      Pss += N*N;

  }

}


void
smooth2( double *EsF, double *Pss, double *wS,
	 int N, int T, int mode, 
	 double *Es, double *Ess )
{
  int t, n, m;
  double ws, Esnp, Essmn;

  /* Move all pointers to end */
  Es  += N*(T-1);
  EsF += N*(T-1);
  if (Ess) Ess += N*N*(T-2);

  /* Initialization @ T-1 */
  for (m=0;m<N;m++)
    Es[m] = EsF[m];
  Es -= N;
  EsF -= N;

  /* One time step difference */
  for (t=T-2;t>=0;t--) {
    
     /* Transition weight */
    if (wS)
      ws = wS[t+1];
    else
      ws = 1;

    for (m=0;m<N;m++)
      Es[m] = -HUGE;

    /* s_{t+1} = n, s_t = m */
    for (n=0;n<N;n++) {
      
      /* Predict: Esnp = Pr(s_{t+1}=n|Y_t) */
      for (Esnp=-HUGE,m=0;m<N;m++)
	Esnp = logSum( Esnp, ws*Pss[n+m*N] + EsF[m] );
      
      /* Find Cs(m,n) = Pr(s_{t+1}=n,s_t=m|Y_T) =
	 Pr(s_{t+1}=n|s_t=m) Pr(s_t=m|Y_t) Pr(s_{t+1}=n|Y_T) / Pr(s_{t+1}=n|Y_t)
      */
      
      /* s_t = m */
      for (m=0;m<N;m++) {
	
	if (finite(Esnp)) {
	  Essmn = ws*Pss[n+m*N] + EsF[m] - Esnp + Es[n+N];
	  Es[m] = logSum( Es[m], Essmn );
	} else
	  Essmn = -HUGE;
	
	if (Ess)
	  Ess[n+m*N] = Essmn;
	
      }

    }

    Es -= N;
    if (Ess)
      Ess -= N*N;
    EsF -= N;

  }

}

void
sparseSmooth2( double *EsF, double *Pss, int *jc_Pss, int *ir_Pss, double *wS,
	       int N, int T, int mode, 
	       double *Es, double *Ess )
{
  int t, n, m, ms, rs, re, mrow;
  double ws, *pss, *Esp, Essmn;

  /* Allocate Esp */
  Esp = (double*) calloc(N,sizeof(double));

  /* Move all pointers to end */
  Es  += N*(T-1);
  EsF += N*(T-1);
  if (Ess) Ess += N*N*(T-2);

  /* Initialization @ T-1 */
  for (m=0;m<N;m++)
    Es[m] = EsF[m];
  Es -= N;
  EsF -= N;

  /* One time step difference */
  for (t=T-2;t>=0;t--) {
    
     /* Transition weight */
    if (wS)
      ws = wS[t+1];
    else
      ws = 1;

    for (m=0;m<N;m++)
      Es[m] = Esp[m] = -HUGE;

    /* Predict */
    for (n=0;n<N;n++) {

      if (finite(EsF[n])) {

	rs  = jc_Pss[n];
	re  = jc_Pss[n+1];
	pss = Pss + rs;

	for (m=rs;m<re;m++,pss++) {
	  mrow = ir_Pss[m];
	  Esp[mrow] = logSum( Esp[mrow], *pss + EsF[n] );
	}

      }

    }

    /* s_{t+1} = m, s_t = n */
    for (n=0;n<N;n++) {
      
      /* Find Cs(m,n) = Pr(s_{t+1}=m,s_t=n|Y_T) =
	 Pr(s_{t+1}=m|s_t=n) Pr(s_t=n|Y_t) Pr(s_{t+1}=m|Y_T) / Pr(s_{t+1}=m|Y_t)
      */

      rs  = jc_Pss[n];
      re  = jc_Pss[n+1];
      pss = Pss + rs;
      
      /* s_{t+1} = m */
      for (m=0,ms=rs;m<N;m++) {
	
	mrow = ir_Pss[ms];
	
	if (finite(Esp[m]) && m==mrow) {
	  Essmn = ws*(*pss) + EsF[n] - Esp[m] + Es[m+N];
	  Es[n] = logSum( Es[n], Essmn );
	  if (ms<re) ms++;
	  pss++;
	} else
	  Essmn = -HUGE;
	
	if (Ess)
	  Ess[m+n*N] = Essmn;
	
      }

    }

    Es -= N;
    if (Ess)
      Ess -= N*N;
    EsF -= N;

  }

  free(Esp);

}


void
forwardFun( double *Pss, int N, obsFun PysFun, int T, double *Ps0, double *wS, 
	    int mode, 
	    double *Es, double *Scale, double *Ess )
{
  double	*EsOld;
  double	ws, Pys, Essmn;
  int		n, m, t;
  
  /* Initialize: t=0 */
  if (wS)
    ws = wS[0];
  else
    ws = 1;
  Scale[0] = -HUGE;
  
  /* Observation + prior */
  for (m=0; m<N; m++) {
    
    /* Observation */
    Pys = PysFun(m,0);
    
    if ( finite(Pys) && finite(Ps0[m]) ) {
      Es[m] = ws*Pys + Ps0[m];
      Scale[0] = logSum( Scale[0], Es[m] );
    } else
      Es[m] = -HUGE;
    
  }
  
  /* Rescale */
  for (m=0; m<N; m++)
    if ( finite(Es[m]) && finite(Scale[0]) )
      Es[m] -= Scale[0];

  /* Update pointers */
  Es    += N;
  
  /* For time steps 1...T-1 */
  for (t=1;t<T;t++) {
    
    Scale[t]	= -HUGE;
    EsOld	= Es - N;
    
    /* Transition & observation weights */
    if (wS) 
      ws = wS[t];
    else
      ws = 1;
    
    /* For each TO state */
    for (m=0; m<N; m++) {
      
      /* Propagation */
      Es[m] = -HUGE;
      
      /* Observation */
      Pys = PysFun(m,t);

      if ( finite(Pys) ) {
	
	/* For each FROM state */
	for (n=0; n<N; n++) {
	  
	  if ( finite(Pss[n*N+m]) && finite(EsOld[n]) )
	    Essmn = EsOld[n] + ws*Pss[n*N+m] + Pys;
	  else
	    Essmn = -HUGE;

	  Es[m] = logSum( Es[m], Essmn );
	  
	  if (Ess)
	    Ess[m+n*N] = Essmn;
	  
	  
	} /* n=0...N-1 */
	
	/* Scale */
	Scale[t] = logSum( Scale[t], Es[m] );
	
      } else {
	
	if (Ess)
	  for (n=0; n<N; n++)
	    Ess[m+n*N] = -HUGE;
	
      }
      
    } /* m=0...N-1 */
    
    /* Rescale */
    for (m=0; m<N; m++) {
      
      if ( finite(Es[m]) && finite(Scale[t]) ) {
	Es[m] -= Scale[t];
	if (Ess)
	  for (n=0; n<N; n++)
	    Ess[m+n*N] -= Scale[t];
      }
      
    }
    
    /* Update pointers */
    Es    += N;
    if (Ess)
      Ess += N*N;
    if (mode)
      Pss += N*N;
    
  }
  
}

void
backwardFun( double *Pss, int N, obsFun PysFun, int T, double *wS, 
	     double *Scale, int mode,
	     double *Beta )
{
  double	*BetaOld, *PysOld, Pobs, ws;
  int		n, m, t;
  
  /* Point to the end of arrays */
  Beta  += N*(T-1);
  BetaOld = Beta + N;
  if (mode)
    Pss += N*N*(T-2);
  
  /* For each time step */
  for (t=T-1; t>=0; t--) {
 
     /* Transition weight */
    if (wS && t<T-1)
      ws = wS[t+1];
    else
      ws = 1;


    /* For each FROM state */
    for (m=0; m<N; m++) {
      
      /* Propagation */
      
      if (t<T-1) {
	
	Beta[m] = -HUGE;
	
	/* For each TO state */
	for (n=0; n<N; n++) {

	  Pobs = PysFun(n,t+1);

	  if ( 
#ifdef CHECK_FINITE
	      finite(Pss[m*N+n]) && finite(BetaOld[n]) && finite(Pobs)
#else
	      1
#endif
	      )
	    Beta[m] = logSum( Beta[m], BetaOld[n] + ws*Pss[m*N+n] + Pobs);

	}
	
	if ( 
#ifdef CHECK_FINITE
	    finite( Beta[m] ) && finite( Scale[t] )
#else
	    1
#endif
	    )
	  Beta[m] -= Scale[t];
	else
	  Beta[m] = -HUGE;
	
      } else {

	Beta[m] = -Scale[t];
	
      }
      
      
    }
    
    Beta    -= N;
    BetaOld -= N;
    if (t<T-1 && mode)
      Pss -= N*N;
    
    
  }
  
}


void
smooth1Fun( double *Es, double *Beta, double *Pss, obsFun PysFun, double *wS,
	    int N, int T, int mode, 
	    double *Cs )
{
  int t, n, m;
  double logVal, Pobs, ws;
  
  /* One time step difference */
  Beta += N;

  for (t=0;t<T-1;t++) {
    
     /* Transition weight */
    if (wS)
      ws = wS[t+1];
    else
      ws = 1;
    

    for (n=0;n<N;n++)
      
      for (m=0;m<N;m++) {

	Pobs = PysFun(m,t+1);

	logVal = Es[n] + ws*Pss[n*N+m] + Beta[m] + Pobs;
	
	if ( finite( logVal ) )
	  Cs[n*N+m] = exp( logVal );
	else
	  Cs[n*N+m] = 0;
      
      }

    Cs    += N*N;
    Es += N;
    Beta  += N;

    if(mode)
      Pss += N*N;

  }

}

/* 
 * Uses sparse Pss structure.
 * If Idx is NULL, does not do backtracing, just returns the best score
 */
void
sparseViterbi( double *Pss, int *jc_Pss, int *ir_Pss, int N,  
	       double *Ps0,
	       double *Pys, double *wS, int T,
	       unsigned int *Idx, double *score )
{
  double	*deltaNew, *deltaOld, *Delta, *pss, *tmp, newVal, ws;
  unsigned int	n, m, *Phi, s;
  int		t, rs, re, nto;
  
  if (Idx) {
    Phi   = calloc(N*T,sizeof(unsigned int));
  }

  Delta = calloc(2*N,sizeof(double));
  deltaNew = Delta;
  deltaOld = Delta+N;

  /* Initialize */
  if (wS)
    ws = wS[0];
  else
    ws = 1;

  for (m=0;m<N;m++) {
    if ( finite( Ps0[m] ) && finite( Pys[m] ) )
      deltaOld[m] = ws*Ps0[m] + Pys[m];
    else
      deltaOld[m] = -HUGE;
  }

  Pys += N;
  if (Idx)
    Phi+=N;

  /* For each time step */
  for (t=1;t<T;t++, Pys+=N) {

    /* Transition weights */
    if (wS)
      ws = wS[t];
    else
      ws = 1;

    /* Initialize each TO state */
    for (m=0; m<N; m++)
      deltaNew[m] = -HUGE;

    /* For each FROM state */
    for (m=0; m<N; m++) {

      if ( finite( deltaOld[m] ) ) {

	/* Start & stop indices of rows in m-th column of Pss */
	rs = jc_Pss[ m ];
	re = jc_Pss[ m+1 ];
	
	/* Pss elements start here */
	pss = Pss + rs;

	/* For each TO state connected to FROM */
	for (n=rs;n<re;n++,pss++) {
	
	  newVal = *(pss)*ws + deltaOld[ m ];
	
	  nto = ir_Pss[n];

	  if ( newVal > deltaNew[ nto ] ) {
	    
	    deltaNew[ nto ] = newVal;
	      
	    if (Idx)
	      Phi[ nto ] = m;
	    
	  }
	
	} /* n = TO states */
	
      } /* finite deltaOld[m] */
	
    } /* m = FROM states */

    if (Idx)
      Phi+=N;
   
    /* For each new state add observations */
    for (m=0;m<N;m++) {
      /* Observation */
      if ( finite(Pys[m]) && finite(deltaNew[m]) )
	deltaNew[m] += Pys[m];
      else
	deltaNew[m] = -HUGE;
    }

    tmp = deltaOld;
    deltaOld = deltaNew;
    deltaNew = tmp;

  } /* t=0:T-1 */

  /* Backtracing */
  score[0] = -HUGE;
  for (m=0;m<N;m++) {
    if (deltaOld[m] > score[0]) {
      score[0] = deltaOld[m];
      if (Idx)
	Idx[T-1] = m + INDEX_OFFSET;
    }
  }

  if (Idx) {

    Phi-=N;
    for(t=T-2;t>=0;t--,Phi-=N) {
      Idx[t] = Phi[ Idx[t+1] - INDEX_OFFSET ] + INDEX_OFFSET;
    }

  }

  /* This is for compatibility only */
  score[0] *= -1.0/T;
  
  free(Delta);
  if (Idx)
    free(Phi);
      
}

void
sparseViterbiFun( double *Pss, int *jc_Pss, int *ir_Pss, int N,  
		  double *Ps0,
		  obsFun PysFun, double *wS, int T,
		  unsigned int *Idx, double *score )
{
  double	*deltaNew, *deltaOld, *Delta, *pss, *tmp, newVal, ws;
  unsigned int	n, m, *Phi, s;
  int		t, rs, re, nto;
  
  if (Idx) {
    Phi   = calloc(N*T,sizeof(unsigned int));
  }

  Delta = calloc(2*N,sizeof(double));
  deltaNew = Delta;
  deltaOld = Delta+N;

  /* Initialize */
  if (wS)
    ws = wS[0];
  else
    ws = 1;

  for (m=0;m<N;m++)
    if ( finite( Ps0[m] ) )
      deltaOld[m] = ws*Ps0[m] + PysFun(m,0);
    else
      deltaOld[m] = -HUGE;
  Phi += N;

  /* For each time step */
  for (t=1;t<T;t++) {

    if (wS)
      ws = wS[t];
    else
      ws = 1;

    /* Initialize each TO state */
    for (m=0; m<N; m++)
      deltaNew[m] = -HUGE;

    /* For each FROM state */
    for (m=0; m<N; m++) {

      if ( finite( deltaOld[m] ) ) {

	/* Start & stop indices of rows in m-th column of Pss */
	rs = jc_Pss[ m ];
	re = jc_Pss[ m+1 ];
	
	/* Pss elements start here */
	pss = Pss + rs;

	/* For each TO state connected to FROM */
	for (n=rs;n<re;n++,pss++) {
	
	  newVal = *(pss)*ws + deltaOld[ m ];
	  	
	  nto = ir_Pss[n];

	  if ( newVal > deltaNew[ nto ] ) {
	    
	    deltaNew[ nto ] = newVal;
	      
	    if (Idx)
	      Phi[ nto ] = m;
	    
	  }
	
	} /* n = TO states */
	
      } /* finite deltaOld[m] */
	
    } /* m = FROM states */

    if (Idx)
      Phi+=N;
   
    /* For each new state add observations */
    for (m=0;m<N;m++) {
      /* Observation */
      if ( finite(deltaNew[m]) )
	deltaNew[m] += PysFun(m,t);
      else
	deltaNew[m] = -HUGE;
    }

    tmp = deltaOld;
    deltaOld = deltaNew;
    deltaNew = tmp;

  } /* t=0:T-1 */

  /* Backtracing */
  score[0] = -HUGE;
  for (m=0;m<N;m++) {
    if (deltaOld[m] > score[0]) {
      score[0] = deltaOld[m];
      if (Idx)
	Idx[T-1] = m + INDEX_OFFSET;
    }
  }

  if (Idx) {

    Phi-=N;
    for(t=T-2;t>=0;t--,Phi-=N) {
      Idx[t] = Phi[ Idx[t+1] - INDEX_OFFSET ] + INDEX_OFFSET;
    }

  }

  /* This is for compatibility only */
  score[0] *= -1.0/T;
  
  free(Delta);
  if (Idx)
    free(Phi);
      
}

#ifdef NEW_SPARSE_FUN

int
sparseViterbiFun( double *Pss, int *jc_Pss, int *ir_Pss, int N,  
		  double *Ps0,
		  obsFun PysFun, 
		  double *wS, int T,
		  unsigned int *Idx, double *score )
{
  double	*deltaNew, *deltaOld, *Delta, *pss, *tmp, newVal, ws;
  unsigned int	n, m, *Phi, s, phiCounter, *curPhi, *ir_Phi, *jc_Phi;
  int		t, rs, re, nto;
  int		avLinkSize;
  
  /* Find average link size - used to allocate memory for Phi */
  avLinkSize = ( (double)jc_Pss[N] ) / N;
  if (avLinkSize<1) avLinkSize = 1;

  if (Idx) {
    phiSize = avLinkSize*T;
    if ( (Phi = malloc(phiSize*sizeof(unsigned int)) ) == NULL )
      return -1;
    if ( (ir_Phi = malloc(phiSize*sizeof(unsigned int)) ) == NULL )
      return -1;
    if ( (jc_Phi = malloc((T+1)*sizeof(unsigned int)) ) == NULL )
      return -1;
    if ( (curPhi = malloc(N*sizeof(unsigned int))) = NULL )
      return -1;
  }

  if ( ( Delta = malloc(2*N*sizeof(double)) ) == NULL )
    return -1;

  deltaNew = Delta;
  deltaOld = Delta+N;

  /* Initialize */
  if (wS)
    ws = wS[0];
  else
    ws = 1;

  for (m=0;m<N;m++)
    if ( finite( Ps0[m] ) )
      deltaOld[m] = ws*Ps0[m] + PysFun(m,0);
    else
      deltaOld[m] = -HUGE;

  /* For each time step */
  for (t=1;t<T;t++) {

    if (wS)
      ws = wS[t];
    else
      ws = 1;

    /* Initialize each TO state */
    for (m=0; m<N; m++)
      deltaNew[m] = -HUGE;

    /* For each FROM state */
    for (m=0; m<N; m++) {

      if ( finite( deltaOld[m] ) ) {

	/* Start & stop indices of rows in m-th column of Pss */
	rs = jc_Pss[ m ];
	re = jc_Pss[ m+1 ];
	
	/* Pss elements start here */
	pss = Pss + rs;

	/* For each TO state connected to FROM */
	for (n=rs;n<re;n++,pss++) {
	
	  newVal = *(pss)*ws + deltaOld[ m ];
	
	  nto = ir_Pss[n];

	  if ( newVal > deltaNew[ nto ] ) {
	    
	    deltaNew[ nto ] = newVal;
	      
	    if (Idx)
	      curPhi[ nto ] = m;
	    
	  }
	
	} /* n = TO states */
	
      } /* finite deltaOld[m] */
	
    } /* m = FROM states */

    /* For each new state add observations */
    for (m=0;m<N;m++) {
      /* Observation */
      if ( finite(deltaNew[m]) )
	deltaNew[m] += PysFun(m,t);
      else
	deltaNew[m] = -HUGE;
    }

    if (Idx) {

      /* Copy curPhi into sparse Phi */
      for (m=0,n=0;m<N;m++) {
	
	if ( finite( deltaNew[m] ) ) {
	  
	  if (phiCounter>=phiSize) {
	    /* Need to reallocate memory for Phi */
	    phiSize += N;
	    if ( (Phi = realloc(Phi,phiSize*sizeof(unsigned int))) == NULL )
	      return -1;
	    if ( (ir_Phi = realloc(ir_Phi,phiSize*sizeof(unsigned int))) 
		 == NULL )
	      return -1;
	  }
	  
	  Phi[phiCounter]    = curPhi[m];
	  ir_Phi[phiCounter] = n;
	  
	  phiCounter++;
	  n++;
	}
	
      }
      
      jc_Phi[t+1] = phiCounter;
      
    }
   
    tmp = deltaOld;
    deltaOld = deltaNew;
    deltaNew = tmp;

  } /* t=0:T-1 */

  /* Backtracing */
  score[0] = -HUGE;
  for (m=0;m<N;m++) {
    if (deltaOld[m] > score[0]) {
      score[0] = deltaOld[m];
      if (Idx)
	Idx[T-1] = m + INDEX_OFFSET;
    }
  }

  if (Idx) {

    for(t=T-2;t>=0;t--) {
      Idx[t] = Phi[ jc_Phi[t] + ir_Phi[ jc_Phi[t] + Idx[t+1] - INDEX_OFFSET ] + INDEX_OFFSET;
    }

  }

  /* This is for compatibility only */
  score[0] *= -1.0/T;
  
  free(Delta);
  if (Idx)
    free(Phi);
      
}

#endif

void
viterbiFun( double *Pss, int N, double *Ps0, obsFun PysFun, double *wS, int T, 
	    int mode,
	    unsigned int *Idx, double *score )
{
  double	*deltaNew, *deltaOld, *Delta, *pss, *tmp, ws;
  unsigned int	n, m, *Phi, s;
  int		t;

  if (Idx) {
    Phi   = calloc(N*T,sizeof(unsigned int));
  }
  Delta = calloc(2*N,sizeof(double));

  deltaNew = Delta;
  deltaOld = Delta+N;

  /* For each time step */
  for (t=0;t<T;t++) {

    if (wS)
      ws = wS[0];
    else
      ws = 1;

    /* For each TO state */
    for (m=0; m<N; m++) {

      /* Propagation */

      if (t>0) {
	
	pss = Pss + m;
	deltaNew[m] = -HUGE;
	
	/* For each FROM state */
	for (n=0; n<N; n++, pss+=N) {
	  if ( finite(*pss) && finite(deltaOld[n]) ) {

	    score[0] = deltaOld[n] + (*pss)*ws;
	    
	    if (score[0] > deltaNew[m]) {
	      deltaNew[m] = score[0];
	      if (Idx)
		Phi[m]      = n;
	    }
	  }
	}
	
      } else {
	
	deltaNew[m] = *(Ps0++)*ws;
      
      }

      /* Observation */
      if ( finite(deltaNew[m]) )
	deltaNew[m] += PysFun(m,t);
      else
	deltaNew[m] = -HUGE;


    }


    tmp = deltaOld;
    deltaOld = deltaNew;
    deltaNew = tmp;

    if(t>0 && mode)
      Pss += N*N;

    if (Idx)
      Phi+=N;

  }

  /* Backtracing */

  score[0] = -HUGE;
  for (m=0;m<N;m++) {
    if (deltaOld[m] > score[0]) {
      score[0] = deltaOld[m];
      if (Idx)
	Idx[T-1] = m + INDEX_OFFSET;
    }
  }

  if (Idx) {

    Phi-=N;
    
    for(t=T-2;t>=0;t--,Phi-=N) {
      Idx[t] = Phi[ Idx[t+1] - INDEX_OFFSET ] + INDEX_OFFSET;
    }
    
  }
  
  /* This is for compatibility only */
  score[0] *= -1.0/T;

  free(Delta);
  if (Idx)
    free(Phi);
      
}


#undef INDEX_OFFSET

void
sparseForward( double *Pss, int *jc_Pss, int *ir_Pss, int N, 
	       double *Pys, int T, double *Ps0, double *wS, 
	       double *Es, double *Scale, double *Ess )
{
  double	*EsOld;
  double	*pss, ws, Essmn;
  int		n, m, t, rs, re, nto;
  
  /* Initialize */
  if (wS)
    ws = wS[0];
  else
    ws = 1;

  Scale[0] = -HUGE;
  for (m=0;m<N;m++) {

    if ( finite(Ps0[m]) && finite( Pys[m] ) )
      Es[m] = ws*Ps0[m] + Pys[m];
    else
      Es[m] = -HUGE;
      
    /* Scale */
    Scale[0] = logSum( Scale[0], Es[m] );
    
  }

  /* Rescale */
  for (m=0; m<N; m++)
    if ( finite(Es[m]) && finite(Scale[0]) )
      Es[m] -= Scale[0];
    else
      Es[m] = -HUGE;

  /* Update pointers */
  Es    += N;
  Pys   += N;
    
  /* For each time step */
  for (t=1;t<T;t++) {

    if (wS)
      ws = wS[t];
    else
      ws = 1;
    
    Scale[t]   = -HUGE;
    EsOld = Es - N;

    for (m=0;m<N;m++) {
      Es[m] = -HUGE;
      if (Ess)
	for (n=0;n<N;n++)
	  Ess[m+n*N] = -HUGE;
    }
    
    /* For each FROM state */
    for (m=0; m<N; m++) {
      
      if ( finite( EsOld[m]) ) {

	/* Start & stop indices of rows in m-th column of Pss */
	rs = jc_Pss[ m ];
	re = jc_Pss[ m+1 ];
	
	/* Pss elements start here */
	pss = Pss + rs;
	
	/* For each TO state */
	for (n=rs; n<re; n++,pss++) {

	  nto = ir_Pss[n];

	  if ( finite(Pys[nto]) ) {
	    
	    Essmn = EsOld[m] + (*pss)*ws + Pys[nto];

	    Es[nto] = logSum( Es[nto], Essmn );

	    if (Ess)
	      Ess[nto+m*N] = Essmn;

	  } /* finite Pys */

	} /* finite EsOld */
	
      } /* TO */
      
    } /* FROM */
    
    /* Scale */
    for (m=0;m<N;m++)
      Scale[t] = logSum( Scale[t], Es[m] );
    
    /* Rescale */
    for (m=0; m<N; m++) {

      if ( finite(Es[m]) )
	Es[m] -= Scale[t];
	if (Ess)
	  for (n=0; n<N; n++)
	    Ess[m+n*N] -= Scale[t];
	
    }

    /* Update pointers */
    Es    += N;
    Pys   += N;
    if (Ess)
      Ess += N*N;
    
  }
  
}


void
sparseBackward( double *Pss, int *jc_Pss, int *ir_Pss, int N, 
		double *Pys, int T, double *wS, double *Scale,
		double *Beta )
{
  double	*BetaOld, *PysOld, *pss, ws;
  int		n, m, t, rs, re, nfrom;
  
  /* Point to the end of arrays */
  Beta  += N*(T-1);
  PysOld  = Pys + N*T;
  BetaOld = Beta + N;
  
  /* For each time step */
  for (t=T-1; t>=0; t--) {

    if (wS && t<T-1)
      ws = wS[t+1];
    else
      ws = 1;

    /* For each TO state */
    for (m=0; m<N; m++) {
      
      /* Propagation */
      
      if (t<T-1) {
	
	Beta[m] = -HUGE;
	
	/* Start & stop indices of columns in m-th row of Pss' */
	rs = jc_Pss[ m ];
	re = jc_Pss[ m+1 ];

	/* Pss elements start here */
	pss = Pss + rs;

	/* For each FROM state */
	for (n=rs; n<re; n++,pss++) {

	  nfrom = ir_Pss[n];

	  if ( finite(BetaOld[nfrom]) && finite(PysOld[nfrom]) )

	      Beta[m] = logSum( Beta[m], BetaOld[nfrom] + ws*(*pss) 
				+ PysOld[nfrom]);

	}
	
	if ( finite( Beta[m] ) && finite( Scale[t] ) )
	  Beta[m] -= Scale[t];
	else
	  Beta[m] = -HUGE;
	
      } else {

	Beta[m] = -Scale[t];
	
      }
      
      
    }
    
    Beta    -= N;
    BetaOld -= N;
    PysOld  -= N;
    
    
  }
  
}

void
sparseSmooth1( double *Es, double *Beta, 
	       double *Pss, int *jc_Pss, int *ir_Pss, 
	       double *Pys, double *wS,
	       int N, int T, 
	       double *Cs )
{
  int t, n, m, rs, re, mrow, ms;
  double logVal, *pss, ws;
  
  /* One time step difference */
  Pys += N;
  Beta += N;

  for (t=0;t<T-1;t++) {
    
    if (wS)
      ws = wS[t+1];
    else
      ws = 1;

    /* Columns of Pss increased by Es */
    for (n=0;n<N;n++) {

      /* Start & stop indices of rows in m-th column of Pss */
      rs = jc_Pss[ n ];
      re = jc_Pss[ n+1 ];
    
      /* Pss elements start here */
      pss = Pss + rs;

      /* Rows of Pss increased by Beta + Pys */
      for (m=0,ms=rs;m<N;m++) {

	mrow = ir_Pss[ms];
	
	if (m==mrow) {
	  
	    logVal = Es[n] + ws*(*pss) + Beta[mrow] + Pys[mrow];
	  
	  if ( finite( logVal ) )
	    Cs[n*N+m] = exp( logVal );
	  else
	    Cs[n*N+m] = 0;
	  
	  if (ms<re)
	    ms++;
	  pss++;
	  
	} else {
	  
	  Cs[n*N+m];
	  
	}
	
      } /* rows */
      
    } /* coulmns */

    Cs    += N*N;
    Es += N;
    Beta  += N;
    Pys   += N;
    
      
  }
  
}


void
sparseForwardFun( double *Pss, int *jc_Pss, int *ir_Pss, int N, 
		  obsFun PysFun, int T, double *Ps0, double *wS, 
		  double *Es, double *Scale, double *Ess )
{
  double	*EsOld;
  double	*pss, ws, Essmn, Pys;
  int		n, m, t, rs, re, nto;
  
  /* Initialize */
  if (wS)
    ws = wS[0];
  else
    ws = 1;

  Scale[0] = -HUGE;
  for (m=0;m<N;m++) {

    /* Observation */
    Pys = PysFun(m,0);

    if ( finite(Ps0[m]) && finite( Pys ) )
      Es[m] = ws*Ps0[m] + Pys;
    else
      Es[m] = -HUGE;
      
    /* Scale */
    Scale[0] = logSum( Scale[0], Es[m] );
    
  }
  
  /* Rescale */
  for (m=0; m<N; m++)
    if ( finite(Es[m]) && finite(Scale[0]) )
      Es[m] -= Scale[0];
    else
      Es[m] = -HUGE;

  /* Update pointers */
  Es    += N;
    
  /* For each time step */
  for (t=1;t<T;t++) {

    if (wS)
      ws = wS[t];
    else
      ws = 1;
    
    Scale[t]   = -HUGE;
    EsOld = Es - N;

    for (m=0;m<N;m++) {
      Es[m] = -HUGE;
      if (Ess)
	for (n=0;n<N;n++)
	  Ess[m+n*N] = -HUGE;
    }
    
    /* For each FROM state */
    for (m=0; m<N; m++) {
      
      if ( finite( EsOld[m]) ) {

	/* Start & stop indices of rows in m-th column of Pss */
	rs = jc_Pss[ m ];
	re = jc_Pss[ m+1 ];
	
	/* Pss elements start here */
	pss = Pss + rs;
	
	/* For each TO state */
	for (n=rs; n<re; n++,pss++) {

	  nto = ir_Pss[n];

	  /* Observation */
	  Pys = PysFun(nto,t);

	  if ( finite(Pys) ) {
	    
	    Essmn = EsOld[m] + (*pss)*ws + Pys;

	    Es[nto] = logSum( Es[nto], Essmn );

	    if (Ess)
	      Ess[nto+m*N] = Essmn;

	  } /* finite Pys */

	} /* finite EsOld */
	
      } /* TO */
      
    } /* FROM */
 
    /* Scale */
    for (m=0;m<N;m++)
      if (finite(Es[m]))
	Scale[t] = logSum( Scale[t], Es[m] );

    /* Rescale */
    for (m=0; m<N; m++) {

      if ( finite(Es[m]) )
	Es[m] -= Scale[t];
	if (Ess)
	  for (n=0; n<N; n++)
	    Ess[m+n*N] -= Scale[t];
	
    }

    /* Update pointers */
    Es    += N;
    if (Ess)
      Ess += N*N;
    
  }
  
}

void
sparseBackwardFun( double *Pss, int *jc_Pss, int *ir_Pss, int N, 
		   obsFun PysFun, int T, double *wS, double *Scale,
		   double *Beta )
{
  double	*BetaOld, *pss, Pobs, ws;
  int		n, m, t, rs, re, nfrom;
  
  /* Point to the end of arrays */
  Beta  += N*(T-1);
  BetaOld = Beta + N;
  
  /* For each time step */
  for (t=T-1; t>=0; t--) {

    if (wS && t<T-1)
      ws = wS[t+1];
    else
      ws = 1;

    /* For each TO state */
    for (m=0; m<N; m++) {
      
      /* Propagation */
      
      if (t<T-1) {
	
	Beta[m] = -HUGE;
	
	/* Start & stop indices of columns in m-th row of Pss' */
	rs = jc_Pss[ m ];
	re = jc_Pss[ m+1 ];

	/* Pss elements start here */
	pss = Pss + rs;

	/* For each FROM state */
	for (n=rs; n<re; n++,pss++) {

	  nfrom = ir_Pss[n];
	  Pobs  = PysFun(nfrom,t+1);

	  if ( finite(BetaOld[nfrom]) && finite(Pobs) )

	    Beta[m] = logSum( Beta[m], BetaOld[nfrom] + ws*(*pss) + Pobs);

	}
	
	if ( finite( Beta[m] ) && finite( Scale[t] ) )
	  Beta[m] -= Scale[t];
	else
	  Beta[m] = -HUGE;
	
      } else {

	Beta[m] = -Scale[t];
	
      }
      
      
    }
    
    Beta    -= N;
    BetaOld -= N;
    
    
  }
  
}

void
sparseSmooth1Fun( double *Es, double *Beta, 
		  double *Pss, int *jc_Pss, int *ir_Pss, 
		  obsFun PysFun, double *wS,
		  int N, int T, 
		  double *Cs )
{
  int t, n, m, rs, re, mrow, ms;
  double logVal, *pss, Pobs, ws;
  
  /* One time step difference */
  Beta += N;

  for (t=0;t<T-1;t++) {
    
    if (wS)
      ws = wS[t+1];
    else
      ws = 1;

    /* Columns of Pss increased by Es */
    for (n=0;n<N;n++) {

      /* Start & stop indices of rows in m-th column of Pss */
      rs = jc_Pss[ n ];
      re = jc_Pss[ n+1 ];
    
      /* Pss elements start here */
      pss = Pss + rs;

      /* Rows of Pss increased by Beta + Pys */
      for (m=0,ms=rs;m<N;m++) {

	mrow = ir_Pss[ms];
	
	if (m==mrow) {

	  Pobs = PysFun(mrow,t+1);
	  
	  logVal = Es[n] + ws*(*pss) + Beta[mrow] + Pobs;
		  
	  if ( finite( logVal ) )
	    Cs[n*N+m] = exp( logVal );
	  else
	    Cs[n*N+m] = 0;
	  
	  if (ms<re)
	    ms++;
	  pss++;
	  
	} else {
	  
	  Cs[n*N+m];
	  
	}
	
      } /* rows */
      
    } /* coulmns */

    Cs    += N*N;
    Es += N;
    Beta  += N;
    
      
  }
  
}

void
forwardFunFun( stateFun PssFun, int N, obsFun PysFun, int T, 
	       double *Ps0, double *wS,
	       double *Es, double *Scale, double *Ess )
{
  double	*EsOld;
  double	ws, Essmn, Pss, Pys;
  int		n, m, t;
  
  /* Initialize: t=0 */
  if (wS)
    ws = wS[0];
  else
    ws = 1;
  Scale[0] = -HUGE;
  
  /* Observation + prior */
  for (m=0; m<N; m++) {
    Pys = PysFun(m,0);
    if ( finite(Pys) && finite(Ps0[m]) ) {
      Es[m] = ws*Pys + Ps0[m];
      Scale[0] = logSum( Scale[0], Es[m] );
    } else
      Es[m] = -HUGE;
  }
  
  /* Rescale */
  for (m=0; m<N; m++)
    if ( finite(Es[m]) && finite(Scale[0]) )
      Es[m] -= Scale[0];

  /* Update pointers */
  Es    += N;
  
  /* For time steps 1...T-1 */
  for (t=0;t<T;t++) {
    
    Scale[t]	= -HUGE;
    EsOld	= Es - N;
    
    /* Transition & observation weights */
    if (wS) 
      ws = wS[t];
    else
      ws = 1;
    
    /* For each TO state */
    for (m=0; m<N; m++) {
      
      /* Propagation */
      Es[m] = -HUGE;

      /* Observation */
      Pys = PysFun(m,t);

      if ( finite(Pys) ) {
	
	/* For each FROM state */
	for (n=0; n<N; n++) {
	  
	  /* Transition */
	  Pss = PssFun(m,n,t);

	  if ( finite(Pss) && finite(EsOld[n]) ) {
	    
	    Essmn = EsOld[n] + ws*Pss + Pys;
	      
	    Es[m] = logSum( Es[m], Essmn );
	  
	    if (Ess)
	      Ess[m+n*N] = Essmn;

	  }
	  
	} /* n=0...N-1 */
		
	/* Scale */
	Scale[t] = logSum( Scale[t], Es[m] );

      } else {

	if (Ess)
	  for (n=0; n<N; n++)
	    Ess[m+n*N] = -HUGE;
	
      }

    } /* m=0...N-1 */
    
    /* Rescale */
    for (m=0; m<N; m++) {
      
      if ( finite(Es[m]) && finite(Scale[t]) ) {
	Es[m] -= Scale[t];
	if (Ess)
	  for (n=0; n<N; n++)
	    Ess[m+n*N] -= Scale[t];
      }
      
    }
    
    /* Update pointers */
    Es    += N;
    if (Ess)
      Ess += N*N;

  }
  
}

void
backwardFunFun( stateFun PssFun, int N, obsFun PysFun, int T, double *wS, 
		double *Scale,
		double *Beta )
{
  double	*BetaOld, *PysOld, Pobs, Pss, ws;
  int		n, m, t;
  
  /* Point to the end of arrays */
  Beta  += N*(T-1);
  BetaOld = Beta + N;
  
  /* For each time step */
  for (t=T-1; t>=0; t--) {
 
    if (wS && t<T-1)
      ws = wS[t+1];
    else
      ws = 1;

    /* For each FROM state */
    for (m=0; m<N; m++) {
      
      /* Propagation */
      
      if (t<T-1) {
	
	Beta[m] = -HUGE;
	
	/* For each TO state */
	for (n=0; n<N; n++) {

	  Pss	= PssFun(n,m,t+1);
	  Pobs	= PysFun(n,t+1);

	  if ( 
#ifdef CHECK_FINITE
	      finite(Pss) && finite(BetaOld[n]) && finite(Pobs)
#else
	      1
#endif
	      )

	    Beta[m] = logSum( Beta[m], BetaOld[n] + ws*Pss + Pobs);
	    
	}
	
	if ( 
#ifdef CHECK_FINITE
	    finite( Beta[m] ) && finite( Scale[t] )
#else
	    1
#endif
	    )
	  Beta[m] -= Scale[t];
	else
	  Beta[m] = -HUGE;
	
      } else {

	Beta[m] = -Scale[t];
	
      }
      
      
    }
    
    Beta    -= N;
    BetaOld -= N;    
    
  }
  
}


void
smooth1FunFun( double *Es, double *Beta, stateFun PssFun, obsFun PysFun, 
	       double *wS, int N, int T,
	       double *Cs )
{
  int t, n, m;
  double logVal, Pobs, Pss, ws;
  
  /* One time step difference */
  Beta += N;

  for (t=0;t<T-1;t++) {
    
    if (wS)
      ws = wS[t+1];
    else
      ws = 1;

    for (n=0;n<N;n++)
      
      for (m=0;m<N;m++) {

	Pobs	= PysFun(m,t+1);
	Pss	= PssFun(m,n,t+1);

	logVal = Es[n] + ws*Pss + Beta[m] + Pobs;
	
	if ( finite( logVal ) )
	  Cs[n*N+m] = exp( logVal );
	else
	  Cs[n*N+m] = 0;
      
      }

    Cs    += N*N;
    Es += N;
    Beta  += N;

  }

}
