/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:43:45 $
 */
 
#ifndef _UTIL_H_
#define _UTIL_H_

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <float.h>

#ifdef __cplusplus
extern "C" {
#endif
  
  typedef double (*stateFun)(int i, int j, int t);
  typedef double (*obsFun)(int m,int t);
  
  /* Utility functions */
  
  extern double	cost( double *Scale, int T);
  extern void	hmm_clean();
  
  /*
   * Basic HMM operators
   * ===================
   *
   * Every operator can be called using four different function calls
   * that depend on the form of state transition matrix Pss and 
   * state observation probabilities Pys.
   * (1) Basic call: full Pss and pre-computed Pys
   * (2) Basic functional call: full Pss and Pys computed on-the-fly
   *     using function PysFun
   * (3) Sparse call: sparse Pss and pre-computed Pys (doesn't make sense?)
   * (4) Sparse functional call: sparse Pss and Pys computed on-the-fly
   *     using function PysFun.
   */

  extern void	forward( double *Pss, int N, double *Pys, int T, 
			 double *Ps0, double *w, int mode,
			 double *Es, double *Scale, double *Ess );
  extern void	forwardFun( double *Pss, int N, obsFun PysFun, int T, 
			    double *Ps0, double *w, int mode,
			    double *Es, double *Scale, double *Ess );
  
  extern void	backward( double *Pss, int N, double *Pys, int T, 
			  double *w, double *Scale,  int mode, 
			  double *Beta );
  extern void	backwardFun( double *Pss, int N, obsFun PysFun, int T, 
			     double *w, double *Scale,  int mode, 
			     double *Beta );
  
  extern void	viterbi( double *Pss, int N, double *Pys, int T, 
			 double *Ps0, double *w, int mode,
			 unsigned int *Idx, double *score );
  extern void	viterbiFun( double *Pss, int N, double *Ps0, obsFun PysFun, 
			    double *w, int T, 
			    int mode,
			    unsigned int *Idx, double *score );
  extern void	viterbiForwardStep( double *Pss, int N, double *Pys, 
				    double *Pin, double *w,
				    double *Pout, int *idx );
  
  extern void	smooth0( double *EsF, double *Beta, double *Scale, int N,
			 int T, double *Es );
  
  extern void	smooth1( double *Es, double *Beta, double *Pss, double *Pys,
			 double *w, int N, int T,  int mode,
			 double *Ess );
  extern void	smooth2( double *EsF, double *Pss, double *wS,
			 int N, int T, int mode, 
			 double *Es, double *Ess );
  extern void	smooth1Fun( double *Es, double *Beta, double *Pss, 
			    obsFun PysFun,
			    double *w, int N, int T,  int mode,
			    double *Ess );
  /* Sparse Pss */
  extern void	sparseSmooth2( double *EsF, 
			       double *Pss, int *jc_Pss, int *ir_Pss, 
			       double *wS, int N, int T, int mode, 
			       double *Es, double *Ess );

  /* Sparse Pss with precomputed observation Pr */
  
  extern void	sparseForward( double *Pss, int *jc_Pss, int *ir_Pss, int N, 
			       double *Pys, int T, double *Ps0, double *w, 
			       double *Es, double *Scale, double *Ess );
  
  extern void	sparseBackward( double *Pss, int *jc_Pss, int *ir_Pss, int N, 
				double *Pys, int T, double *w, double *Scale,
				double *Beta );
  
  extern void	sparseSmooth1( double *Es, double *Beta, 
			       double *Pss, int *jc_Pss, int *ir_Pss, 
			       double *Pys, double *w,
			       int N, int T, 
			       double *Ess );

  extern void	sparseViterbi( double *Pss, int *jc_Pss, int *ir_Pss, int N,  
			       double *Ps0,
			       double *Pys, double *w, int T,
			       unsigned int *Idx, double *score );
  
  /* Sparse Pss with functional call for observation Pr evaluation */
  
  extern void	sparseForwardFun( double *Pss, int *jc_Pss, int *ir_Pss, 
				  int N, obsFun PysFun, int T, double *Ps0, 
				  double *w, 
				  double *Es, double *Scale, double *Ess );
  
  extern void	sparseBackwardFun( double *Pss, int *jc_Pss, int *ir_Pss, 
				   int N, obsFun PysFun, int T, double *w, 
				   double *Scale,
				   double *Beta );
  
  extern void	sparseSmooth1Fun( double *Es, double *Beta, 
				  double *Pss, int *jc_Pss, int *ir_Pss, 
				  obsFun PysFun, double *w,
				  int N, int T, 
				  double *Ess );

  extern void	sparseViterbiFun( double *Pss, int *jc_Pss, int *ir_Pss, int N,
				  double *Ps0,
				  obsFun PysFun, 
				  double *w, int T,
				  unsigned int *Idx, double *score );
  
  /* Functional form of state & observation distributions */
  
  extern void	forwardFunFun( stateFun PssFun, int N, obsFun PysFun, int T, 
			       double *Ps0, double *w,
			       double *Es, double *Scale, double *Ess );
  
  extern void	backwardFunFun( stateFun PssFun, int N, obsFun PysFun, int T, 
				double *w, double *Scale,
				double *Beta );
  
  extern void	smooth1FunFun( double *Es, double *Beta, stateFun PssFun, 
			       obsFun PysFun, double *w, int N, int T,
			       double *Ess );
  
  
#ifdef __cplusplus
	   }
#endif

#endif /* _UTIL_H_ */
