/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:43:49 $
 */
 
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "hmm_util.h"

#define GET_TOTAL_ELEMENTS(a)  mxIsSparse(a)?mxGetNzmax(a):((mxGetM(a)*mxGetN(a)))

/* Input Arguments */

#define	PSS_IN	prhs[0]
#define PYS	prhs[1]
#define PS0_IN	prhs[2]
#define W	prhs[3]
mxArray		*PSS;
mxArray		*PS0;

/* Output Arguments */

#define SCORE	plhs[0]
#define IDX	plhs[1]



void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Pss, *Pys, *Ps0, *w, *Score, *Idx;
  unsigned int	*tmpIdx;
  int		N, T, t, dims[2], *ir = NULL, *jc = NULL;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 3)  {
    mexErrMsgTxt("Requires at least three input arguments.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("Requires one or two output arguments.");
  }
    
  /* Duplicate parameters that need to be converted to log scale */
  PSS = mxDuplicateArray( PSS_IN );
  PS0 = mxDuplicateArray( PS0_IN );

  /* Get dimsnesions */
  N = mxGetM( PSS );
  T = mxGetN( PYS );

  if ( mxGetN( PSS) != mxGetM( PYS ) ) {
    mexErrMsgTxt("Pss and Pys dimensions do not agree.");
  }    

  /* Get input pointers */
  Pss = mxGetPr( PSS );
  if (mxIsSparse( PSS )) {
    ir  = mxGetIr( PSS );
    jc  = mxGetJc( PSS );
  }
  Pys = mxGetPr( PYS );
  Ps0 = mxGetPr( PS0 );
  if (nrhs==3)
    w = NULL;
  else
    w = mxGetPr( W );

  /* Convert to log scale */
  lin2log( Pss, GET_TOTAL_ELEMENTS(PSS) );
  lin2log( Ps0, GET_TOTAL_ELEMENTS(PS0) );

  /* Create outputs */
  IDX = mxCreateDoubleMatrix(1, T, mxREAL);  /* Should use int array */
  SCORE = mxCreateDoubleMatrix(1, 1, mxREAL);

  /* Create tmp index array */
  tmpIdx = mxCalloc(T,sizeof(unsigned int)); /* This shoud go away if IDX is int */

  /* Assign pointers to the various parameters */
  Idx = mxGetPr( IDX );
  Score = mxGetPr( SCORE );

  /* Call viterbi function */
  if ( ir && jc ) {
    sparseViterbi( Pss, jc, ir, N, Ps0, Pys, w, T, tmpIdx, Score );
  } else
    viterbi(Pss, N, Pys, T, Ps0, w, 0, tmpIdx, Score);

  /* Copy tmpIdx into Idx */
  for (t=0;t<T;t++) Idx[t] = tmpIdx[t];

  /* Free tmpIdx */
  mxFree(tmpIdx);

  return;
}
