function [score,is] = hmm_vite(Pss,Pys,Ps0,w)
%
% hmm_vite -- HMM Viterbi decoding
%
% Usage:
%   [score,is] = hmm_vite(Pss,Pys,Ps0,w)
% Input:
%   Pss		state transition probabilities [ to x from ]
%   Pys		evaluated observation log probabilities [ state x time ]
%   Ps0		initial state probabilities [ state x 1 ]
%   w		state transition probability weights (default 1)
% Output:
%   score	-log Pr( observation sequence, decoded state sequence )
%   idx		decoded state sequence
%
% Note:
%   (1) Viterbi decoding finds a sequence of states 'idx' that maximizes
%       log Pr( observation sequence, state sequence )
%
%   (2) Observation log probabilities are defined as
%
%		Pys = log Pr( observation @ t | state @ t ).
%
% See also:
%   hmm, hmm_fwd, hmm_bwd, hmm_smt, hmm_infr, hmm_sim
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:02 $

% MEX file uses likelihoods, .m uses probabilities

  
Pys = exp(Pys);

T = size(Pys,2);
N = size(Pss,1);

if (nargin<4)
  w = ones(1,T);
end

is = zeros(1,T);
phi = zeros(N,T);

for t=1:T,
  if (t==1),
    delta0 = Ps0;
  else
    [delta0,phi(:,t)]  = max( Pss.^w(t) .* delta0(:,ones(N,1))', [], 2 );
  end
  delta0 = delta0.*Pys(:,t);
end

% Backtracking
[score,is(T)] = max(delta0);
for t=T-1:-1:1,
  is(t) = phi(is(t+1),t+1);
end

score = -max(log(delta0))/T;

