% Linear Dynamic System  (LS)
% ===========================
% 
% Model:
%  
%   x_t = A_t x_{t-1} + v_t + u_t,      E[v_t] = 0, E[v_t v_t'] = Q
%   y_t = C_t x_t + w_t,                E[w_t] = 0, E[w_t w_t'] = R
%                                       E[v_t w_t'] = 0
%   Initial condition:
%   E[x_0] = x_0, E[x_0 x_0'] = Q0
% 
% Notation:
% 
%   x_t         system state at time 't'
%   y_t         observation at time 't'
%   u_t         deterministic input at time 't'
%   N           dim(x_t)
%   M           dim(y_t)
%   A           state transition matrix
%   Q           state noise variance
%   C           observation matrix
%   R           observation noise variance
%   x0          initial state mean
%   Q0          initial state variance
%
% Functions:
% 
%   lds_sim      sample from LS
%   lds_infr     inference in LS
%   lds_learn    learn LS parameters
%   lds_init     initialize LS parameters
%   lds_fwd      filtering in LS
%   lds_smt      fixed interval smoothing in LS
%
% Note:
%
%   lds_fwd and lds_smt accept three types of input parameters:
%   (1) static parametrs, e.g., A_t = A [NxN]
%   (2) time varying parameters, e.g., A_t = A(:,:,t) [NxNxT]
%   (3) parameter function calls, e.g., A_t = 'A_fun', where function A =
%       A_fun(t,x) 
%
% Related toolkits:
%
%   mog         mixture of Gaussians toolkit
%   fa          factor analizer toolkit
%   hmm         hidden Markov models toolkit
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:20 $
