% LS demo script

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:21 $

clear all
echo on

global LS_VERBOSE
LS_VERBOSE = 1;

% Create a LDS that aorresponds to a constant acceleration
% continuous-time models:
%   d/dt(dx/dt) = a + v
% Assume that sampling time is 0.1s, acceleration noise variance,
% E[n^2], is 5, measurement noise variance is 3, intitial condition
% is 0, and a is gravitational constant (9.81m/s^2)

dT = 0.1;
a  = 9.81;
n  = 5;
v  = 3;

[A,C,Q,R,x0] = lds_gen('newton',2,dT,a,n,v,[0;0]);

% I.e., the (zero-order hold) discretized system is
% x_{t+1} = A x_t + B a + Q v
% y_t = C x_t + R w

printmat(A,'A'); printmat(C,'C'); printmat(Q,'Q');

pause

% Simulate S sequences of random length (uniformly distributed btw 100-110)
S = 10;
for s=1:S,
  T = 100+round(rand(1)*10);
  [y{s},x{s}] = lds_sim(A,C,Q,R,x0,Q,T,[],rand(1)*1e3);
end
  
% Assume some arbitrary prior distribution on sequences
pY = rand(1,S); pY = pY/sum(pY);

% Plot one of the data sequences
plot(y{1});

pause

% Now, try to learn those model parameters from the simulated data set y

% Pick some initial model parameters
[Ai,Ci,Qi,Ri,x0i] = lds_gen('newton',2,.1,1,1,1,[0;0]);
printmat(Ai,'Ai'); printmat(Qi,'Qi'); printmat(Ri,'Ri');

pause

% Try Improving these estimates using regular LDS EM
[A1,C1,Q1,R1,x01,Q01,cs] = lds_learn(y, Ai,Ci,Qi,Ri,x0i,Qi,1e-3);

% There seems to be a problem with convergence! The learned parameters are
printmat(A1,'A1'); printmat(C1,'C1')

% ...clearly, of different form from what is needed, say
printmat(A,'A'); printmat(C,'C');

pause

% The reason is that we must set the paramater maximization function to reflect
% the model structure of the problem (the structure of parameters)
global LS_MAX_FUN
LS_MAX_FUN = 'lds_max_newton';

% Current version of 'lds_max_newton' requires that one knows the
% sampling period
global LS_MAX_NEWTON
LS_MAX_NEWTON = dT;

% Previously, the maximization function was 'lds_max', the general (and
% default) maximization function of 'lds_learn'. See 'help lds_learn'

pause

% With the new parameter maximization function set (lds_max_nexton)
% parameters are learned as...
[A1,C1,Q1,R1,x01,Q01,cs] = lds_learn(y, Ai,Ci,Qi,Ri,x0i,Qi,1e-3);

% ... resulting in the following learned parameters
printmat(A1,'A1'); printmat(Q1,'Q1'); printmat(R1,'R1');

% Compare them to the original parameters
printmat(A,'A'); printmat(Q,'Q'); printmat(R,'R');


echo off;



