function [ex,vx,cx,cost] = lds_f_lin( exp,vxp, y, u, A, C, Q, R )
%
% lds_f_lin -- one step filter
%
% Usage:
%   [ex,vx,cx] = lds_f_lin( exp,vxp, y, u, A, C, Q, R )
% Input:
%   exp		E[x_{t-1}|y_{t-1}]
%   vxp		E[x_{t-1} x_{t-1}'|y_{t-1}]
%   y		y_t
%   u		u_t
%   A		A_t
%   C		C_t
%   Q		Q_t
%   R		R_t
% Output:
%   ex		E[x_t|y_t]
%   vx		E[x_t x_t'|y_t]
%   cx		E[x_t x_{t-1}'|y_t]
%   cost	
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:21 $

[M,N] = size(C);

% Predict
exP = A*exp + u;
cxP = A*vxp;
vxP = A*vxp*A' + Q;
 
% Filter
dy  = y - C*exP;

O  = C*vxP*C' + R;
[invO,detO] = sinv(O);

err = dy'*invO*dy;
K   = vxP*C'*invO; 

ex = exP + K*dy;
vx = ( eye(N) - K*C )*vxP;
cx = ( eye(N) - K*C )*cxP;

cost = ( log(detO) + err )/2;
% cost = ( M*log(2*pi) + log(detO) + err )/2;

