/* $Revision: 2.1 $ $Date: 2000/06/01 14:54:34 $ */

#include "mex.h"
#include "lds_util.h"
#include <string.h>

#define GET_TOTAL_ELEMENTS(a)  (mxGetM(a) * mxGetN(a)) 

/* For MATLAB 5.1 */
#ifndef mxArrayToString(a)
char*
mxArrayToString(const mxArray* a)
{
  double *pa;
  int la;
  char *ps;

  pa = mxGetPr(a);
  la = GET_TOTAL_ELEMENTS(a);

  if ( (ps = (char*) malloc(sizeof(char)*la)) == NULL )
    mexErrMsgTxt("Cannot copy array to string.");
    
  memcpy(pa,ps,la*sizeof(char));

  return ps;
}
#endif

/* Input Arguments */

#define Y	prhs[0]
#define	A	prhs[1]
#define C	prhs[2]
#define Q	prhs[3]
#define R	prhs[4]
#define x0	prhs[5]
#define Q0	prhs[6]
#define U	prhs[7]
#define W	prhs[8]

/* Output Arguments */

#define Cost	plhs[0]
#define Ex	plhs[1]
#define	Exx	plhs[2]	
#define Exx1	plhs[3]
#define Err	plhs[4]

int	N, M, T, nDim, dims[3];
mxArray	*output_array[1], *input_array[2];
mxArray	*time, *state;
char	*A_fun_name = NULL,  *Q_fun_name = NULL, *C_fun_name = NULL, 
  *R_fun_name = NULL;
int	A_fixed = 0, Q_fixed = 0, C_fixed = 0, R_fixed = 0;
double	*pA, *pC, *pQ, *pR, *px0, *pQ0;

static double*
A_fun_ext(int t, double *vec)
{
  memcpy(mxGetPr(state), vec, N*sizeof(double));
  *mxGetPr(time) = t;

  mexCallMATLAB(1, output_array, 2, input_array, A_fun_name);
  return mxGetPr(output_array[0]);
}

static double*
Q_fun_ext(int t, double *vec)
{
  memcpy(mxGetPr(state), vec, N*sizeof(double));
  *mxGetPr(time) = t;

  mexCallMATLAB(1, output_array, 2, input_array, Q_fun_name);
  return mxGetPr(output_array[0]);
}

static double*
C_fun_ext(int t, double *vec)
{
  memcpy(mxGetPr(state), vec, N*sizeof(double));
  *mxGetPr(time) = t;

  mexCallMATLAB(1, output_array, 2, input_array, C_fun_name);
  return mxGetPr(output_array[0]);
}

static double*
R_fun_ext(int t, double *vec)
{
  memcpy(mxGetPr(state), vec, N*sizeof(double));
  *mxGetPr(time) = t;

  mexCallMATLAB(1, output_array, 2, input_array, R_fun_name);
  return mxGetPr(output_array[0]);
}

static double*
A_fun_arr(int t, double *vec)
{
  if (A_fixed)
    return pA;
  else
    return ( pA+N*N*t );
}

static double*
Q_fun_arr(int t, double *vec)
{
  if (Q_fixed)
    return pQ;
  else
    return ( pQ+N*N*t );
}

static double*
C_fun_arr(int t, double *vec)
{
  if (C_fixed)
    return pC;
  else
    return ( pC+N*M*t );
}

static double*
R_fun_arr(int t, double *vec)
{
  if (R_fixed)
    return pR;
  else
    return ( pR+M*M*t );
}


void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*pU, *pW, *pY;
  double	*pEx, *pExx, *pExx1, *pCost, *pErr;
  double	alpha, errThd;
  sysFun	A_fun, C_fun, Q_fun, R_fun;
  
  /* Check for proper number of arguments */
  
  if (nrhs < 7)  {
    mexErrMsgTxt("Requires at least seven input arguments.");
  } else if (nlhs > 5) {
    mexErrMsgTxt("Requires no more than five output arguments.");
  }
  
  N = mxGetM( x0 );
  T = mxGetN( Y );
  M = mxGetM( Y );

  /* Initialize global vars */
  A_fixed = Q_fixed = C_fixed = R_fixed = 0;
  A_fun_name = Q_fun_name = C_fun_name = R_fun_name = NULL;

  /* Initialize arrays */
  time  = mxCreateDoubleMatrix(1,1,mxREAL);
  state = mxCreateDoubleMatrix(N,1,mxREAL);
   
  input_array[0] = time;
  input_array[1] = state;

  /* Get input pointers */
  pA = mxGetPr( A );
  pQ = mxGetPr( Q );
  pC = mxGetPr( C );
  pR = mxGetPr( R );
  pY = mxGetPr( Y );
  pQ0 = mxGetPr( Q0 );
  px0 = mxGetPr( x0 );

  /* Select calling functions */
  if (mxIsChar(A)) {
    A_fun_name  = mxArrayToString(A);
    A_fun = A_fun_ext;
  } else {
    if ( GET_TOTAL_ELEMENTS(A) != N*N*T && GET_TOTAL_ELEMENTS(A) != N*N )
      mexErrMsgTxt(" A dimension does not agree.");
    if ( GET_TOTAL_ELEMENTS(A) == N*N )
      A_fixed = 1;
    A_fun = A_fun_arr;
  }

  if (mxIsChar(C)) {
    C_fun_name  = mxArrayToString(C);
    C_fun = C_fun_ext;
  } else {
    if ( GET_TOTAL_ELEMENTS(C) != N*M*T && GET_TOTAL_ELEMENTS(C) != N*M )
      mexErrMsgTxt(" C dimension does not agree.");
    if ( GET_TOTAL_ELEMENTS(C) == N*M )
      C_fixed = 1;
    C_fun = C_fun_arr;
  }

  if (mxIsChar(Q)) {
    Q_fun_name  = mxArrayToString(Q);
    Q_fun = Q_fun_ext;
  } else {
    if ( GET_TOTAL_ELEMENTS(Q) != N*N*T && GET_TOTAL_ELEMENTS(Q) != N*N )
      mexErrMsgTxt(" Q dimension does not agree.");
    if ( GET_TOTAL_ELEMENTS(Q) == N*N )
      Q_fixed = 1;
    Q_fun = Q_fun_arr;
  }

  if (mxIsChar(R)) {
    R_fun_name  = mxArrayToString(R);
    R_fun = R_fun_ext;
  } else {
    if ( GET_TOTAL_ELEMENTS(R) != M*M*T && GET_TOTAL_ELEMENTS(R) != M*M )
      mexErrMsgTxt(" R dimension does not agree.");
    if ( GET_TOTAL_ELEMENTS(R) == M*M )
      R_fixed = 1;
    R_fun = R_fun_arr;
  }

  /* Deterministic input */
  if (nrhs>7) {
    pU = mxGetPr( U );
    if ( GET_TOTAL_ELEMENTS(U) != N*T )
      mexErrMsgTxt(" U dimension does not agree.");
  } else
    pU = NULL;
  
  /* Output weights */
  if (nrhs>8) {
    pW = mxGetPr( W );
    if ( GET_TOTAL_ELEMENTS(W) != T )
      mexErrMsgTxt(" W dimension does not agree.");
  } else
    pW = NULL;

  nDim	= 3;
  dims[0] = N; dims[1] = N; dims[2] = T;

  /* Create outputs */
  Ex	= mxCreateDoubleMatrix(N, T, mxREAL);
  Exx	= mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL);
  dims[2] = T-1;
  Exx1	= mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL);
  Cost	= mxCreateDoubleMatrix(1, 1, mxREAL);
  Err   = mxCreateDoubleMatrix(1, T, mxREAL);


  /* Assign pointers to various parameters */
  pEx	= mxGetPr( Ex );
  pExx	= mxGetPr( Exx );
  pExx1	= mxGetPr( Exx1 );
  pCost	= mxGetPr( Cost );
  pErr  = mxGetPr( Err );

  /* Call forward function */
  if ( A_fixed && C_fixed && Q_fixed && R_fixed ) {
   
    /* Time invariant system */
    lds_forward( pA,pC,pQ,pR,px0,pQ0, pU,pW, pY, N,M,T, 
		 pEx,pExx,pExx1,pCost,pErr );
    
  } else {
    
    /* Time varying system */
    lds_tv_forward( A_fun, C_fun, Q_fun, R_fun, px0, pQ0, 
		    pU, pW, pY, N,M, T,
		    pEx, pExx, pExx1,pCost,pErr );
    
  }

  /* Clean up */

  if (A_fun_name) mxFree(A_fun_name);
  if (C_fun_name) mxFree(C_fun_name);
  if (Q_fun_name) mxFree(Q_fun_name);
  if (R_fun_name) mxFree(R_fun_name);

  lds_clean();

  return;
}
