function [cost,ex,vx,cx] = lds_infr(Y,A,C,Q,R,x0,Q0,u,w)
%
% lds_infr -- linear dynamic system (lds) inference
%
% Usage:
%   [cost,ex,vx,cx] = lds_infr(Y,A,C,Q,R,x0,Q0,U,W)
% Input:
%   Y		observations
%   A		state transition matrix
%   C		observation matrix
%   Q		state noise covariance
%   R		observation noise covariance
%   Q0		initial state noise covariance
% Output:
%   cost	-log{P(Y)}/T
%   ex		E[x_t|O]
%   vx		Var[x_t|O]
%   cx		Covar[x_t,x_(t-1)|O]
% Global variables:
%   LS_INFERENCE_MODE	'exact' -- exact inference (default)
%			'fwd'   -- filtering
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:22 $

global LS_INFERENCE_MODE
if (~isempty(LS_INFERENCE_MODE))
  inferMode = LS_INFERENCE_MODE;
else
  inferMode = 'exact';
end

[M,N] = size(C);
T = size(Y,2);

if (nargin<8)
  u = zeros(N,T);
end

if (nargin<9)
  w = ones(1,T);
end

switch( lower(inferMode) ),
  
 case 'exact',
  
  [cost,ex,vx,cx] = lds_smt(Y,A,C,Q,R,x0,Q0,u,w);
  
 case 'fwd',
  
  [cost,ex,vx,cx] = lds_fwd(Y,A,C,Q,R,x0,Q0,u,w);
  
 otherwise,
  
  error( sprintf('Unknown LS_INFERENCE_MODE option %s.', upper(inferMode)) );
  
end

