function [a,c,q,r,x0,q0] = lds_max_ar( Eyy,Eyx,Exx,ExxD,ExxT,Exx1,Ex0,Exx0, ...
				      T, a,c,q,r,x0,q0)
%
% lds_max_ar -- maximization of LS model parameters
%              state transition is modeled as K-th order AR system
%
% See also:
%   lds_learn
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:23 $

global LS_MIN_VAR
global LS_NL_A LS_NL_C LS_NL_Q LS_NL_R LS_NL_X0

if (isempty(LS_MIN_VAR))
   tol = 1e-3;
else
   tol = LS_MIN_VAR;
end


[M,N] = size(Eyx); K = N/M;
t0 = 1:M;
t1 = M+1:N;

c = [ eye(M) zeros(M,N-M) ];

if (isempty(LS_NL_R))
  r = ( Eyy - c*Eyx' - Eyx*c' + c*Exx*c' )/T;
  r = diag(diag(r));
  r = lim_var( r, tol );
end

a(t0,:) = ExxD(t0,:)*sinv( Exx - ExxT );
a(t1,:) = eye(N-M,N);

if (isempty(LS_NL_X0))
  x0 = Ex0;
end


if (isempty(LS_NL_Q))
  qc = ( Exx - Exx1 - a*ExxD' - ExxD*a' + a*(Exx-ExxT)*a' )/(T-1);
  qc = diag(diag(qc));
  qc = lim_var( qc, tol );
  q  = zeros(N); q(t0,t0) = qc(t0,t0);
  
  q0 = Exx1 - Ex0*x0' - x0*Ex0' + x0*x0';
  q0 = diag(diag(q0));
  q0 = lim_var( q0, tol );
end

