function [a,c,q,r,x0,q0]= ...
    lds_max_newton(Eyy,Eyx,Exx,ExxD,ExxT,Exx1,Ex0,Exx0, T, a,c,q,r,x0,q0)
%
% lds_max_newton -- maximization of LS model parameters
%                  system is zero-order hold discretized version of
%                  continuous time system with Newtonian dynamics
%                  
% See also:
%   lds_learn
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:23 $

global LS_MIN_VAR
global LS_MAX_NEWTON

if (isempty(LS_MIN_VAR))
  tol = 1e-3;
else
  tol = LS_MIN_VAR;
end

if (isempty(LS_MAX_NEWTON))
  dT = 1;
else
  dT = LS_MAX_NEWTON;
end
  
M = size(Eyx,1);
N = ( size(Eyx,2) - 1 ) / M ;
O = N+1;

% Generic newtonian motion params
[an,cn,qn] = lds_gen('newton',N,dT,1,1,1,[],[]);

% Plant
a = an;
a(1:N,O) = ( ExxD(1:N,O) - an(1:N,1:N)*( Exx(1:N,O) - ExxT(1:N,O) ) ) * ...
    sinv( Exx(O,O) - ExxT(O,O) );

% Initial condition
x0 = Ex0;

% Measurement noise
r = ( Eyy - c*Eyx' - Eyx*c' + c*Exx*c')/T;
r = lim_var( r, tol );

% Plant noise
qf = trace( sinv(qn)*( Exx - Exx1 - a*ExxD' - ExxD*a' + a*(Exx-ExxT)*a' )) ...
    / ( T*N );
q = qf*qn;
% q = lim_var( q, tol );

q0 = trace( sinv(qn)*(Exx1 - Ex0*x0' - x0*Ex0' + x0*x0') ) / N;
q0 = qf*qn;
% q0 = lim_var( q0, tol );


