function [exN,vxN,cxN] = lds_s_lin( ex,vx, exf,vxf,cxf, vxfp, A, Q )
%
% lds_s_lin -- one step smoother
%
% Usage:
%   [ex,vx,cx] = lds_s_lin( exp,vxp, exf,vxf,cxf, vxfp, A, Q )
% Input:
%   exp		E[x_{t+1}|y_T]
%   vxp		E[x_{t+1} x_{t+1}'|y_T]
%   exf		E[x_t|y_t]
%   vxf		E[x_t x_t'|y_t]
%   cxf		E[x_{t+1} x_t'|y_{t+1}]
%   vxfp	E[x_{t+1} x_{t+1}'|y_{t+1}]
%   Ap		A_{t+1}
%   Qp		Q_{t+1}
% Output:
%   exN		E[x_t|Y_T]
%   vxN		E[x_t x_t'|y_T]
%   cxN		E[x_{t+1} x_t'|y_T]
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:24 $

exP = A*exf;
vxP = A*vxf*A' + Q;

J = vxf*A'*pinv(vxP);

exN = exf + J*(ex - exP);
vxN = vxf + J*(vx - vxP)*J';
cxN = cxf + (vx - vxfp)*sinv(vxfp)*cxf; 
