function [exN,vxN,cxN] = lds_s_lin_2( ex,vx,cx, exf,vxf, vxf1, A, Q, Ap, Qp )
%
% lds_s_lin_2 -- one step smoother
%
% Usage:
%   [exN,vxN,cxN] = lds_s_lin_2( ex,vx,cx, exf,vxf, vxfp, A, Q, Ap, Qp )
% Input:
%   ex		E[x_{t+1}|y_T]
%   vx		E[x_{t+1} x_{t+1}'|y_T]
%   cx		E[x_{t+1} x_t'|y_T]
%   exf		E[x_t|y_t]
%   vxf		E[x_t x_t'|y_t]
%   vxfp	E[x_{t-1} x_{t-1}'|y_{t-1}]
%   A		A_{t+1}
%   Q		Q_{t+1}
%   Ap		A_t
%   Qp		Q_t
% Output:
%   exN		E[x_t|Y_T]
%   vxN		E[x_t x_t'|y_T]
%   cxN		E[x_t x_{t-1}'|y_T]
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:24 $

exP = A*exf;
vxP = A*vxf*A' + Q;
J   = vxf*A'*pinv(vxP);

vxP1 = Ap*vxf1*Ap' + Qp;
J1   = vxf1*Ap'*pinv(vxP1);

exN = exf + J*(ex - exP);
vxN = vxf + J*(vx - vxP)*J';
cxN = vxf*J1' + J*(cx - A*vxf)*J1'; 
