function [y,x] = lds_sim(A,C,Q,R,x0,Q0,T,u,seed)
%
% lds_sim -- Simulation of a linear dynamic system
% 
%  Usage:
%    [y,x] = lds_sim(A,C,Q,R,x0,Q0,T,u,seed)
%  Input:
%   A		state transition matrix
%   Q		state noise variance
%   C		observation matrix
%   R		observation noise variance
%   x0		initial state mean
%   Q0		initial state variance
%   T		sequence duration
%   u		deterministic input (default no input)
%   seed	random generator seed (default 0)
%  Output:
%   y		observation sequence
%   x		state sequence
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:25 $

N = size(A,1);

if (isstr(C))
   M = size(feval(C,zeros(N,1),0),1);
else
   M = size(C,1);
end

if (nargin<8 | isempty(u))
   u = zeros(N,T);
end

if (nargin<9)
   seed=0;
end

[Uq,DIAGq,UTq]    = svd(Q);
[Uq0,DIAGq0,UTq0] = svd(Q0);
[Ur,DIAGr,UTr]    = svd(R);

randn('seed',seed);

x(:,1) = x0 + u(:,1) + Uq0*sqrt(DIAGq0)*randn(N,1);
y(:,1) = C*x(:,1) + Ur*sqrt(DIAGr)*randn(M,1);

for t=2:T,
   
  x(:,t) = A*x(:,t-1) + u(:,t) + Uq*sqrt(DIAGq)*randn(N,1);
  
  y(:,t) = C*x(:,t) + Ur*sqrt(DIAGr)*randn(M,1);
  
end
