function [cost,ex,vx,cx,err] = lds_smt(y,A,C,Q,R,x0,Q0,u,w)
%
% lds_smt -- linear dynamic system (lds) inference
%
% Usage:
%   [cost,ex,vx,cx,err] = lds_smt(Y,A,C,Q,R,x0,Q0,U,W)
% Input:
%   Y		observations
%   A		state transition matrix
%   C		observation matrix
%   Q		state noise covariance
%   R		observation noise covariance
%   x0		initial state mean
%   Q0		initial state noise covariance
%   U		deterministic system input  (default 0)
%   W		observation weights, R_t = R / W_t (default 1)
% Output:
%   cost	-log{P(Y)}/T
%   ex		E[x_t|O]
%   vx		Var[x_t|O]
%   cx		Covar[x_t,x_(t-1)|O]
%   err		innovation error
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:25 $

[M,T] = size(y);

[M,N,L] = lds_dim(A,C,Q,R,x0,Q0);

if (nargin<8)
  u = [];
end

if (nargin<9)
  w = ones(1,T);
end

% Forward pass (Kalman filter)
[cost,exf,vxf,cxf,err] = lds_fwd(y,A,C,Q,R,x0,Q0,u,w);

% Backward pass

% Initialize
ex(:,T)   = exf(:,T);
vx(:,:,T) = vxf(:,:,T);

for t=T-1:-1:1,
  
  % Parameters
  [a,c,q,r] = lds_param_fun(A,C,Q,R,t);
  
  % Smooth
  [ex(:,t),vx(:,:,t),cx(:,:,t)] = ...
      lds_s_lin( ex(:,t+1),vx(:,:,t+1), exf(:,t),vxf(:,:,t), cxf(:,:,t), ...
		vxf(:,:,t+1), a, q );

end
