function [cost,es,cs] = mc_infr(y,P,p0)
%
% mc_infr -- Markov chain inference
%
% Usage:
%   [cost,es,cs] = mc_infr(y,P,p0)
%
% Input:
%   y		observation sequence
%   P		log of state transition probability matrix
%   p0		log of initial state distribution
%
% Output:
%   cost	log Pr( observation sequence )
%   es		E[ s_t ] = Pr( s_t )
%   cs		E[ s_t s_{t-1}' ] = Pr( s_t s_{t-1}' )
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:08 $

N = size(P,1);
T = size(y,2);

% Find es from y
base = eye(N);
es = base(:,y);

Pf = P;
ni = find(isinf(P));
Pf(ni) = -realmin;

pf0 = p0;
ni = find(isinf(p0));
pf0(ni) = -realmin;

cost = - ( sum( sum( es(:,2:T) .* ( Pf * es(:,1:T-1) ), 1) ) + ...
	 pf0(y(1)) ) / T;

if (nargout>2)
  % Find cs from es
  for t=1:T-1,
    cs(:,:,t) = es(:,t+1)*es(:,t)';
  end
end

