function [cost,P,p0,Es,Cs] = mc_init(y,N,type,param)
%
% mc_init -- initialize Markov chain parameters
%
% Usage:
%   [cost,P,p0,es,cs] = mc_init(y,N,type,param)
% Input:
%   y		data sequences
%   N		number of states
%   type	'hlr' - left-to-right (hard)
%		'slr' - left-to-right (soft)
%		'lt'  - lower triangular P
%		'full'- full P
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:09 $

if (~iscell(y))
  y = {y};
end

S = length(y);

if (nargin<4)
  param = [];
end

% Initialize statistics
[Ess,Es0,T] = mc_ics(N);

for k = 1:S,
  
  % Generate default sequences
  [es,cs] = mc_gen( N, size(y{k},2), type, param );
  
  % Update statistics
  [Ess,Es0,T] = mc_ucs(es,cs,Ess,Es0,T);
  
  if (nargout>3)
    Es{k} = es;
  end
  
  if (nargout>4)
    Cs{k} = cs;
  end
  
end

% Maximize
[P,p0] = mc_max( Ess, Es0, T, S, [], [] );  

% Cost after maximization
cost = mc_cost( Ess, Es0, P, p0, T, S);
