function [T,mT] = mc_stat(P,p0)
%
% mc_stat -- Markov chain temporal statistics
%
% Usage:
%  [T,mT] = mc_stat(P,p0)
% Input:
%  P		state transition probability matrix
%  p0		initial state distribution vector
% Output:
%  T		average chain duration
%  mT		average state durations
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:10 $

N = size(P,1);

if (nargin<2)
  p0 = [1;zeros(N-1,1)];
end

% Works only on left-to-right HMMs for now?
% See E. Seneta, "Non-negative matrices & Markov chains," p. 122-131

% Average absorption time of states (1 ... N-1) into N

mT = ones(1,N-1)*inv(eye(N-1)-P(1:N-1,1:N-1));

T  = mT*p0(1:N-1);

