% MOG demo script

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:13 $

echo on
warning off
clf

% This is a MOG

Q = [.4 .3 .3]';
C = [0 10 -10];
R = [4 4 4 ];

pause

% Simulate S=50 samples these parameters
S = 50;

% Assume some arbitrary distribution of samples
pY = rand(1,S); pY = pY/sum(pY);

% ...then simulate the model using default random seed of '0'
[y,s] = mog_sim(Q,C,R,S,rand(1)*1e3);

% plot states 'y'
subplot(311); stairs(y); title('Observations - y');
subplot(312); stairs(s); title('States - s');

pause

% Inference: probabilities of mixtures for each observation
% esc(c,1,t) = Pr( mixture = c | observation @ t)
[cost,ec] = mog_infr( y, Q,C,R );

% plot probabilities of hidden states (mixture components)
subplot(313); stairs(ec'); axis([0 S -.1 1.1]);
title('Probabilities of hidden states');
legend('Pr(s_t=1)','Pr(s_t=2)','Pr(s_t=3)');

pause

% Estimate parameters of this MOG given the data

% Enable printing of scores
global MOG_VERBOSE
MOG_VERBOSE = 1;

% Initial values are used to get parameter dims
[Q_0,C_0,R_0] = mog_init(y, 3,0);

% Learn parameters
[Q_1,C_1,R_1] = mog_learn(y, Q_0,C_0,R_0, 1e-8, pY);

printmat(Q_1); printmat(C_1); printmat(R_1);
pause

% Now change inference mode to approximate ('wta') and learn model parameters
global MOG_INFERENCE_MODE
MOG_INFERENCE_MODE = 'wta';

[Q_1,C_1,R_1] = mog_learn(y, Q_0,C_0,R_0, 1e-8, pY);

printmat(Q_1); printmat(C_1); printmat(R_1);
pause
echo off

