/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:42:07 $
 */
 
/* $Revision: 2.0 $ */
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "mog_util.h"

#define GET_TOTAL_ELEMENTS(a)  (mxGetM(a) * mxGetN(a)) 

/* Input Arguments */

#define	X_IN	prhs[0]		/* ldim x time */
#define	C_IN_L	prhs[1]		/* c x s */
#define M_IN	prhs[2]		/* ldim x c x s */
#define R_IN	prhs[3]		/* ldim x c x s */
mxArray		*C_IN;


/* Output Arguments */

#define	P_OUT	plhs[0]		/* s x time */
#define PC_OUT	plhs[1]		/* c x s x time */


void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{
  double	*p, *pc;
  double	*x, *c, *m, *r;
  int		N, M, S, T;
  int		fullR, nDim, dims[3];
  
  /* Check for proper number of arguments */
  
  if (nrhs != 4) {
    mexErrMsgTxt("MOG requires four input arguments.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("MOG requires two output argument.");
  }
  
    /* Duplicate parameters that are to be converted to log space */
  C_IN = mxDuplicateArray( C_IN_L );

  /* Get dimensions */
  N = mxGetM(X_IN);
  T = mxGetN(X_IN);
  
  M = mxGetM(C_IN);
  S = mxGetN(C_IN);
  
  /* Is R full? */
  if ( mxGetNumberOfDimensions(M_IN)>2 )
    fullR = mxGetNumberOfDimensions(R_IN) - mxGetNumberOfDimensions(M_IN);
  else
    fullR = GET_TOTAL_ELEMENTS(R_IN) - GET_TOTAL_ELEMENTS(M_IN); 

  /* mexPrintf("%d %d %d %d\n", N, M, S, T); */
 
  if (GET_TOTAL_ELEMENTS(M_IN) != (N*M*S)) {
     mexErrMsgTxt("MOG M dimension does not agree.");
  }
  
  if (fullR) {
    if (GET_TOTAL_ELEMENTS(R_IN) != (N*N*M*S)) {
      mexErrMsgTxt("MOG full R dimension does not agree.");
    } 
  } else {
    if (GET_TOTAL_ELEMENTS(R_IN) != (N*M*S)) {
      mexErrMsgTxt("MOG diagonal R dimension does not agree.");
    } 
  }
  
  /* Get input pointers */
  x = mxGetPr(X_IN);
  c = mxGetPr(C_IN);
  m = mxGetPr(M_IN);
  r = mxGetPr(R_IN);
  
  /* Convert to log space */
  lin2log(c,GET_TOTAL_ELEMENTS(C_IN));

  /* Create outputs */
  P_OUT = mxCreateDoubleMatrix(S, T, mxREAL);
  nDim = 3;
  dims[0] = M; dims[1] = S; dims[2] = T;
  PC_OUT = mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL);
  
  
  /* Assign pointers to the various parameters */
  p = mxGetPr(P_OUT);
  pc = mxGetPr(PC_OUT);

  /* Call C function */
  if (fullR) {
    if( mog_full( x, N, T, c, m, r, M, S, p, pc ) < 0 )
      mexWarnMsgTxt("MOG R is not positive definite.");
  } 
  else
    mog_diag( x, N, T, c, m, r, M, S, p, pc );

  return;

  mog_clean();

}
