/*
*	$Revision: 1.5 $ $Date: 2000-03-03 14:32:04-05 $
 */
 
/* $Revision: 1.5 $ */
#include <math.h>
#include "mex.h"
#include "../emf/math_util.h"
#include "mog_util.h"

#define GET_TOTAL_ELEMENTS(a)  (mxGetM(a) * mxGetN(a)) 

/* Input Arguments */

#define	X_IN	prhs[0]		/* ldim x time */
#define	Q_IN	prhs[1]		/* c x s */
#define C_IN	prhs[2]		/* ldim x c x s */
#define R_IN	prhs[3]		/* ldim x c x s */


/* Output Arguments */

#define	GQ_OUT	plhs[0]		/* c x s  */
#define	GC_OUT	plhs[1]		/* ldim x c x s  */
#define	GR_OUT	plhs[2]		/* ldim x c x s  */


void mexFunction(
                 int nlhs,       mxArray *plhs[],
                 int nrhs, const mxArray *prhs[]
		 )
{
  double	*ecst, *est, *logPst, *logPcst, *gq, *gc, *gr, gs[1];
  double	*x, *p, *c, *m, *r, *pC, pP[1] = {1};
  int		i,j, N, M, S, T;
  int		nDim, fullR, dims[4];
  
  /* Check for proper number of arguments */
  
  if (nrhs != 4) {
    mexErrMsgTxt("MOG_SMT requires four input arguments.");
  } else if (nlhs > 4) {
    mexErrMsgTxt("MOG_SMT requires three output arguments.");
  }
  
  
  N = mxGetM(X_IN);
  T = mxGetN(X_IN);
  
  M = mxGetM(Q_IN);
  S = mxGetN(Q_IN);
  
  /* mexPrintf("%d %d %d %d\n", N, M, S, T); */
 
  /* Is R full? */
  if ( mxGetNumberOfDimensions(C_IN)>2 )
    fullR = mxGetNumberOfDimensions(R_IN) - mxGetNumberOfDimensions(C_IN);
  else
    fullR = GET_TOTAL_ELEMENTS(R_IN) - GET_TOTAL_ELEMENTS(C_IN); 

  /* Check mean size */
  if (GET_TOTAL_ELEMENTS(C_IN) != (N*M*S)) {
     mexErrMsgTxt("MOG M dimension does not agree.");
  }
  
  /* Check variance size */
  if (fullR && GET_TOTAL_ELEMENTS(R_IN) != (N*M*S) )
      mexErrMsgTxt("MOG R dimension does not agree.");
  else
    if (GET_TOTAL_ELEMENTS(R_IN) != (N*M*S))
      mexErrMsgTxt("MOG R dimension does not agree.");
  
  /* Create outputs */
  GQ_OUT = mxCreateDoubleMatrix(M, S, mxREAL);
  nDim = 3; dims[0] = N; dims[1] = M; dims[2] = S;
  GC_OUT = mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL);
  if (fullR) {
    nDim = 4; dims[0] = N; dims[1] = N; dims[2] = M; dims[3] = S;
  }
  GR_OUT = mxCreateNumericArray(nDim, dims, mxDOUBLE_CLASS, mxREAL);
   
  /* Assign pointers to the various parameters */
  gq = mxGetPr(GQ_OUT);
  gc = mxGetPr(GC_OUT);
  gr = mxGetPr(GR_OUT);

  x = mxGetPr(X_IN);
  c = mxGetPr(Q_IN);
  m = mxGetPr(C_IN);
  r = mxGetPr(R_IN);
  
  /* Temporary storage for logPst, logPcst, ecst, est, pC, & pP */
  logPst  = mxCalloc(S*T, sizeof(double));
  logPcst = mxCalloc(M*S*T, sizeof(double));
  ecst = mxCalloc(M*S*T, sizeof(double));
  est = mxCalloc(S*T, sizeof(double));  
  pC  = mxCalloc(S*M, sizeof(double));

  /* Make internal versions of exp(c) and exp(p) */
  for (j=0; j<M; j++)
    pC[j] = c[j];

  /* Convert to log space */
  lin2log(c,GET_TOTAL_ELEMENTS(C_IN));

  /* Call mog function. It uses likelihoods. */
  if (fullR)
    if ( mog_full( x, N, T, c, m, r, M, S, logPst, logPcst ) < 0 )
      mexErrMsgTxt("MOG_SMT R is not positive definite.");
  else
    mog_diag( x, N, T, c, m, r, M, S, logPst, logPcst );

  /* Call mog_smt */
  mog_smt( logPst, logPcst, S, M, T, est, ecst);

  /* Call mog_grad. It uses probabilities. */
  mog_grad( x, ecst, est, N, M, S, T, pP, pC, m, r, fullR, gs, gq, gc, gr );

  /* Free temp storage */
  free(logPst);
  free(logPcst);
  free(ecst);
  free(est);
  free(pC);

  mog_clean();

  return;

}

