function [cost,ec] = mog_infr(y,Q,C,R)
%
% mog_infr -- mixture of Gaussians inference
%
% Usage:
%   [cost,ec] = mog_infr(y,Q,C,R,mode)
%
% Input:
%   y		observation sequence
%   Q		mixture weights
%   C		means
%   R		variances
%
% Output:
%   cost	log Pr( observation sequence )
%   ec		E[ c ]
%
% Global settings:
%   MOG_INFERENCE_MODE	'exact'	-- exact inference (default)
%			'wta'	-- winner-takes-all
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:14 $

global MOG_INFERENCE_MODE

if (~isempty(MOG_INFERENCE_MODE))
  inferMode = MOG_INFERENCE_MODE;
else
  inferMode = 'exact';
end

switch(lower(inferMode))
  
 case 'exact',
  
  [cost,ec] = mog_smt(y,Q,C,R);

 case 'wta',
  
  N = size(Q,1);
  basis = eye(N);
  [py,pyc] = mog_eval(y,Q,C,R);
  [cy,idx] = max(pyc,[],1);
  ec = basis(:,idx);
  cost = - cy(:)';
  
 otherwise,
  
  error( sprintf('Unknown MOG_INFERENCE_MODE option %s.', upper(inferMode)) );
  
end
