function [Q,C,R,ec] = mog_init( y, M, d, pY)
%
% mog_init -- initialize MOG parameters
%
% Usage:
%   [Q,C,R,ec] = mog_init( y, M, vmode, pY)
%
% Input:
%   y		list of observation sequences
%   M		number of mixture components
%   vmode	Gaussian mixture mode: 0 - diagonal, 1 - full
%   pY		distribution of observation sequences
%
% Output:
%   Q		mixture weights
%   C		mixture means
%   R		mixture variances
%   ec		E[ c_t | observations ]
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:14 $

% Dimensions
[D,S] = size(y);

if nargin<5 | isempty(pY),
  pY = ones(1,S)/S;
end

% Basis vectors for mixtures
baseM = eye(M);


% Classify data
if (M>1)
  [tmp,tmp,tmp,tmp,tmp,map] = vq_flat( y, M );
  ec(1:M,1:S) = baseM(:,map);
else
  ec(1:M,1:S) = 1;
end
      
% Initialize scores
[Ec,Eyc,Eyyc,L] = mog_ics(M,D,d);

% Update scores
[Ec,Eyc,Eyyc,L] = mog_ucs(y,ec,Ec,Eyc,Eyyc,L,pY);

% Maximize
[Q,C,R] = mog_max(Ec,Eyc,Eyyc,[],[],[],L);
