function [y,c] = mog_sim(Q,C,R,T,seed)
%
% mog_sim -- simulate mixture of Gaussians
%
% Usage:
%   [y,c] = mog_sim(Q,C,R,L,seed)
% Input:
%   Q		mixture weights distribution
%   C		mixture means
%   R		mixture covariances
%   L		number of simulated data points
%   seed	random seed (default 0)
% Output:
%   y		observation vector
%   c		mixture component vector
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:15 $

if (nargin<5)
  seed = 0;
end
rand('seed',seed);
randn('seed',seed);

if (nargin<4)
  T = 1;
end

M = size(Q,1);
D = size(C,1);
d = ndims(R)-ndims(C);

if d
  for c = 1:M,
    [Ur(:,:,c),DIAGr(:,c),tmp] = svd(R(:,:,c));
  end
end

cumQ = cumsum(Q,1);

rM = rand(1,T);
rY = randn(D,T);

% Pick mixture component
c = sum( rM(ones(M,1),:) > cumQ(:,ones(1,T)), 1 ) + 1;

% Simulate y (only diag R)
for t=1:T,
  
  if d
    y(:,t) = C(:,c(t)) + ...
	Ur(:,:,c(t))*sqrt( DIAGr(:,c(t)) ).* rY(:,t);
  else
    y(:,t) = C(:,c(t)) + sqrt( R(:,c(t)) ).* rY(:,t);
  end
  
end
