function [cost,ec] = mog_smt(y,Q,C,R)
%
% mog_smt -- MOG state smoothing
%
% Estimates of states and (states,mixtures) for a set of data vectors
% 'y' and MOG model with parameters (Q,C,R)
%
% Usage:
%   [cost,ec] = mog_smt(y,Q,C,R)
% Input:
%   y		data
%   Q		mixture weights
%   C		mixture means
%   R		mixture variances (diagonal or full)
% Output:
%   ec		E[ mixture | data, model ]
%
% See also:
%   mog
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:15 $

M = size(Q,1);
S = size(y,2);

[logPs,logPcs] = mog_eval(y,Q,C,R);

ec = zeros(M,S);
ni = find(~isinf(logPs));

ec(:,ni) = exp( permute(logPcs(:,:,ni),[1 3 2]) - logPs(ones(M,1),ni) );

cost = -logPs;
