/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:42:07 $
 */
 
#include "../emf/math_util.h"

#undef DEBUG

#define log_2pi	1.837877066409345

void
mog_clean()
{
  emf_clean();
}

double 
logGaussDiag(double x, double m, double r)
{
  double del, retVal = 0;

  if (r>0) {
    del = x - m;
    retVal =  -( del*del/r + log_2pi + log(r) )/2. ;
  } else {
    if (x == m)
      retVal = 0;
    else
      retVal = -HUGE;
  }

  return retVal;
}

double 
logGaussFull(double *x, double *m, double *l, double *p, int N)
  /* l is lower triangular matrix from Cholesky decomp.
     p is diagonal from Cholesky decomp. */
{
  int i, j;
  double retVal = 0;
  double sum;

  for (i=0;i<N;i++){
    
    if ( p[i]>0 ) {

      for (sum=0,j=0;j<=i;j++) {
	sum += l[i+j*N]*( x[j]-m[j] );
      }
    
      retVal += -( ( sum*sum + log_2pi )/2 - log(p[i]) );
      
    } else {
      
      if ( x[i] == m[i] )
	retVal += -HUGE;
      else
	retVal += 0;
    
    }

  }

  return retVal;

}


void
mog_diag( double *x, int N, int T, double *c, double *m, double *r, int M, 
	  int S, 
	  double *p, double *pc )
{
  int t, s, mix, dim;
  double tmp;
  double *cPtr, *mPtr, *rPtr;

  for (t=0;t<T;t++) {

    cPtr = c;
    mPtr = m;
    rPtr = r;
    
    for (s=0; s<S; s++) {
      
      *p = -HUGE;

      for (mix=0; mix<M; mix++) {

        if ( finite(*cPtr) ) {
          

          tmp = *cPtr;
          for (dim=0; dim<N; dim++)
            tmp += logGaussDiag(*(x+dim),*(mPtr++),*(rPtr++));
          
          if (pc) *pc = tmp;
          *p = logSum( *p, tmp );
          
        } else {
          
          if (pc) *pc = -HUGE;
          mPtr += N;
          rPtr += N;
          
        }
        
        cPtr++;
        if (pc) pc++;
        
      } /* for mix */
      
      p++;
      
    } /* for s */

    
    x += N;
    
  } /* for t */

}

int
mog_full(double *x, int N, int T, double *c, double *m, double *r, 
	 int M, int S, 
	 double *p, double *pc)
{
  int i,j, t, s, mix, dim;
  double tmp, retVal = 0;
  double *cPtr, *mPtr, *lPtr, *dlPtr;
  double *invL;
  double *diagL;

  /* First find inverse of r */
  invL  = (double*)malloc(N*N*M*S*sizeof(double));
  diagL = (double*)malloc(N*M*S*sizeof(double));

  lPtr = invL;
  dlPtr = diagL;

  for (s=0;s<S;s++)
    for (mix=0;mix<M;mix++) {
      
      /* Copy upper triangle of r(mix,s) into invL */
      for (i=0;i<N;i++)
	for (j=0;j<N;j++)
	  lPtr[i+j*N] = r[i+j*N];

      /* Find Cholesky factorization in the lower triangle of invL */
      retVal = choldc(lPtr, N, dlPtr);

      /* Find inverse of the lower triangular matrix and leave it in invL */
      invchol(lPtr, dlPtr, N);
     
      lPtr  += N*N;
      r     += N*N;
      dlPtr += N;

    }
  
  for (t=0;t<T;t++) {
    
    cPtr = c;
    mPtr = m;
    lPtr = invL;
    dlPtr = diagL;
    
    for (s=0; s<S; s++) {
      
      *p = -HUGE;
      
      for (mix=0; mix<M; mix++) {
	
        if ( finite(*cPtr) ) {
          
          tmp = *cPtr + logGaussFull(x,mPtr,lPtr,dlPtr,N);
          
          if (pc) *pc = tmp;
          *p = logSum( *p, tmp );
          
        } else {
          
          if (pc) *pc = -HUGE;
          
        }

	mPtr  += N;
	lPtr  += N*N;
	dlPtr += N;
        
        cPtr++;
        if (pc) pc++;
        
      } /* for mix */
      
      p++;
      
    } /* for s */
    
    x += N;
    
  } /* for t */

  return retVal;
}



void
mog_smt( double *logPst, double *logPcst, int S, int M, int T, 
	 double *Est, double *Ecst )
{
  int t, mix,s;
  double logPt;

  for(t=0;t<T;t++, logPst += S) {

    /* Find log(P[t]) */
    logPt = -HUGE;
    for (s=0;s<S;s++)
      logPt = logSum( logPt, logPst[s] );
    
    /* Find Es[t], Ecs[t] */
    for (s=0;s<S;s++, logPcst += M) {

      if (Est)
	*(Est++) = exp( logPst[s] - logPt );
      
      if (Ecst)
	for (mix=0;mix<M;mix++)
	  *(Ecst++) = exp( logPcst[mix] - logPt );
    }
    
  }
}

void
mog_grad( double *x, double *Ecst, double *Est, int N, int M, int S, int T, 
	  double *p, double *c, double *m, double *r, int fullR,
	  double *gS, double *gC, double *gM, double *gR )
{
  
  int t,s,i,j,mix, sizeR;
  double *cVal, *mVal, *rVal, *gRVal, *gMVal, *gCVal, *gCSum, gSSum = 0;
  double *diff;

  gCSum = (double*)calloc(S, sizeof(double));
  diff  = (double*)calloc(N, sizeof(double));

  /* Size of R */
  if (fullR)
    sizeR = N*N;
  else
    sizeR = N;

  /* Gradients */
  for(t=0;t<T;t++,x+=N,Est+=S) {
    
    /* Init pointers */
    cVal=c, mVal=m, rVal=r, gMVal=gM, gRVal=gR, gCVal=gC;

    for(s=0; s<S; s++, Ecst+=M, cVal+=M, gCVal+=M) {

      for(mix=0;mix<M;mix++, mVal+=N, rVal+=sizeR, gMVal+=N, gRVal+=sizeR) {
	
	for(i=0;i<N;i++) {
	  
	  diff[i] = ( x[i] - mVal[i] );
	  
	  /* Mean */
	  gMVal[i] += diff[i] / rVal[i] * Ecst[mix] / T;
	  
	}
	
	/* Variance */
	if (fullR) {
	  for (i=0;i<N;i++)
	    for (j=0;j<N;j++)
	      gRVal[i+j*N] += ( rVal[i+j*N] - diff[i]*diff[j] ) * 
		Ecst[mix] / (2.*T);
	} else
	  for (i=0;i<N;i++)
	    gRVal[i] += ( rVal[i] - diff[i]*diff[i] ) * Ecst[mix] / (2.*T);
	
	/* Mixture coeff */
	if ( c[mix] )
	  gCVal[mix] += Ecst[mix] / c[mix] / T;
	
      }
      
      /* State */
      if ( p[s] ) {
	gS[s] += Est[s] / p[s] / T;
      }
      
    }
    
  }
  
  /* Project gC & gS onto constraint surfaces */
  for(s=0; s<S; s++) {
    gSSum += gS[s]/S;
    for(mix=0; mix<M; mix++)
      gCSum[s] += gC[s*M + mix]/M;
  }

  for(s=0; s<S; s++) {
    gS[s] -= gSSum;
    for(mix=0; mix<M; mix++)
      gC[s*M + mix] -= gCSum[s];
  }

  free(gCSum);
  free(diff);

}




