/*
*	$Revision: 2.0 $ $Date: 2000/04/24 17:40:52 $
 */
 
/* $Revision: 2.0 $ */
#include <math.h>
#include "mex.h"
#include "vq_util.h"

#define GET_TOTAL_ELEMENTS(a)  (mxGetM(a) * mxGetN(a)) 

/* Input Arguments */

#define	VEC	prhs[0]
#define CMEAN	prhs[1]
#define CVAR	prhs[2]
#define CVTYPE	prhs[3]
#define CM	prhs[4]

/* Output Arguments */

#define COST	plhs[0]
#define	CMAP	plhs[1]
#define CCOST	plhs[2]
#define CSIZE	plhs[3]


void 
mexFunction(
	    int nlhs,       mxArray *plhs[],
	    int nrhs, const mxArray *prhs[]
	    )
{
  double	*Vec, *Cost, *CMean, *CVar, *CMap, *CSize, *CCost, *tmp;
  double	*iCVar, dCVar;
  int		i, N, L, Nc, CVType = NULLC, maxIter = 100, minClustSize = 1;
  int		*cmap, *csize, dims[3];
  
  /* Check for proper number of arguments */
  
  if (nrhs < 2)  {
    mexErrMsgTxt("Requires at least two arguments.");
  }
  
  /* Get input pointers */
  Vec = mxGetPr( VEC );
  CMean = mxGetPr( CMEAN );

  N = mxGetM( VEC );
  L = mxGetN( VEC );
  Nc= mxGetN( CMEAN );

  if (mxGetM( CMEAN ) != N)
    mexErrMsgTxt("Data and parameter dimensions disagree.");

  if (nrhs>=3)
    CVar = mxGetPr( CVAR );
  else
    CVar = NULL;
  
  if (nrhs>=4) {
    tmp = mxGetPr( CVTYPE );
    CVType = (int) tmp[0];
    if (CVType<NULLC || CVType>INVFULLC)
      mexErrMsgTxt("Unsupported covariance type.");     
  }
  
  if (nrhs>=5) {
    tmp = mxGetPr( CM );
    minClustSize = (int) tmp[0];
  }
  
  if (CVar)
    if ( GET_TOTAL_ELEMENTS( CMEAN ) == GET_TOTAL_ELEMENTS( CVAR ) ) {
      if ( (CVType != DIAGC) && (CVType != INVDIAGC) ) {
	if (nrhs>3)
	  mexWarnMsgTxt("Covariance dimensions and type disagree.");
	CVType = DIAGC;
      }
    } else if ( GET_TOTAL_ELEMENTS( CMEAN ) == GET_TOTAL_ELEMENTS( CVAR )/N ) {
      if ( (CVType != FULLC) && (CVType != INVFULLC) ) {
	if (nrhs>3)
	  mexWarnMsgTxt("Covariance dimensions and type disagree.");
	CVType = FULLC;
      }
    } else
      mexErrMsgTxt("Covariance dimensions and type disagree."); 
  
  /* Convert FULLC into INVFULLC because Distance() only supports INVFULLC */
  if (CVType==FULLC) {
    iCVar = mxCalloc(Nc*N*N,sizeof(double));
    for(i=0;i<Nc;i++)
      sinv(CVar+i*N*N,N,iCVar+i*N*N,&dCVar);
    CVar = iCVar;
    CVType = INVFULLC;
  }
      
  /* Set covariance params */
  setCov(CVar,CVType,N);
	
  /* Create outputs */
  COST = mxCreateDoubleMatrix(1,1, mxREAL);
  Cost = mxGetPr( COST );

  CCOST = mxCreateDoubleMatrix(1,Nc, mxREAL);
  CCost = mxGetPr( CCOST );
  
  CSIZE = mxCreateDoubleMatrix(1,Nc, mxREAL);
  CSize = mxGetPr( CSIZE );
  
  CMAP = mxCreateDoubleMatrix(1,L, mxREAL);
  CMap = mxGetPr( CMAP );

  /* Allocate tmp data storage */
  cmap  = mxCalloc(L,sizeof(int));
  csize = mxCalloc(Nc,sizeof(int));

  /* Call FlatCluster function */
  AllocateVectors( Vec, N, L,
		   CMean,
		   CCost, csize, cmap, Nc, minClustSize, Cost);


  /* Copy tmp to out */
  for (i=0;i<L;i++)
    CMap[i] = cmap[i]+1;
  for (i=0;i<Nc;i++)
    CSize[i] = csize[i];

  /* Free tmp data storage */
  mxFree(cmap);
  mxFree(csize);
  if (CVType==FULLC) mxFree(iCVar);

  return;
}
